@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
+ *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +33,7 @@
 #include "TreeSearchWidget.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
+#include "playlist3-playlist-editor.h"
 
 static void pl3_file_browser_edit_columns(void);
 
@@ -51,7 +52,6 @@ static void pl3_file_browser_save_myself(void);
 static int pl3_file_browser_add_go_menu(GtkWidget *menu);
 static void pl3_file_browser_activate(void);
 static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
-static gboolean pl3_file_browser_button_press_event(GtkWidget *but, GdkEventButton *event);
 static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp);
 static void pl3_file_browser_add_selected(void);
 static void pl3_file_browser_replace_selected(void);
@@ -148,7 +148,13 @@ static void pl3_file_browser_dir_row_activated(GtkTreeView *tree, GtkTreePath *t
 	else
 		gtk_tree_view_expand_row(tree,tp,FALSE);
 }
-
+static void playtime_changed(GmpcMpdDataModel *model, gulong playtime)
+{
+    if(pl3_cat_get_selected_browser() == file_browser_plug.id)
+    {
+        playlist3_show_playtime(playtime);
+    }
+}
 
 static void pl3_file_browser_init()
 {
@@ -159,6 +165,8 @@ static void pl3_file_browser_init()
     int pos;
 
 	pl3_fb_store2 = gmpc_mpddata_model_new();
+    gmpc_mpddata_model_disable_image(GMPC_MPDDATA_MODEL(pl3_fb_store2));
+    g_signal_connect(G_OBJECT(pl3_fb_store2), "playtime_changed", G_CALLBACK(playtime_changed), NULL);
 
 
     pos = cfg_get_single_value_as_int_with_default(config, "file-browser", "pane-pos", 150);
@@ -203,11 +211,10 @@ static void pl3_file_browser_init()
     
 	/* set up the tree */
 	pl3_fb_tree= gmpc_mpddata_treeview_new("file-browser",TRUE, GTK_TREE_MODEL(pl3_fb_store2));
-
+    gmpc_mpddata_treeview_enable_click_fix(GMPC_MPDDATA_TREEVIEW(pl3_fb_tree));
   /* setup signals */
 	g_signal_connect(G_OBJECT(pl3_fb_tree), "row-activated",G_CALLBACK(pl3_file_browser_row_activated), NULL);
 	g_signal_connect(G_OBJECT(pl3_fb_tree), "button-release-event", G_CALLBACK(pl3_file_browser_button_release_event), NULL);
-	g_signal_connect(G_OBJECT(pl3_fb_tree), "button-press-event", G_CALLBACK(pl3_file_browser_button_press_event), NULL);
 	g_signal_connect(G_OBJECT(pl3_fb_tree), "key-press-event", G_CALLBACK(pl3_file_browser_playlist_key_press), NULL);
 
 	/* set up the scrolled window */
@@ -336,7 +343,7 @@ static void pl3_file_browser_add(GtkWidget *cat_tree)
 	GtkTreePath *path;
 	gint pos = cfg_get_single_value_as_int_with_default(config, "file-browser","position",2);
 	playlist3_insert_browser(&iter, pos);
-	gtk_tree_store_set(pl3_tree, &iter, 
+	gtk_list_store_set(GTK_LIST_STORE(pl3_tree), &iter, 
 			PL3_CAT_TYPE, file_browser_plug.id,
 			PL3_CAT_TITLE, _("File Browser"),
 			PL3_CAT_INT_ID, "/",
@@ -574,7 +581,7 @@ static void pl3_file_browser_fill_tree(GtkWidget *tree,GtkTreeIter *iter, GtkTre
 
 
 
-static int pl3_file_browser_cat_popup(GtkWidget *tree, GdkEventButton *event, gpointer user_data)
+static int pl3_file_browser_cat_popup(GtkWidget *wid, GdkEventButton *event, gpointer user_data)
 {
     GtkWidget *menu;
     if(event->button == 3){
@@ -674,6 +681,8 @@ static void pl3_file_browser_selected(GtkWidget *container)
     gtk_container_add(GTK_CONTAINER(container), pl3_fb_vbox);
     gtk_widget_grab_focus(pl3_fb_tree);
     gtk_widget_show(pl3_fb_vbox);
+
+    playlist3_show_playtime(gmpc_mpddata_model_get_playtime(GMPC_MPDDATA_MODEL(pl3_fb_store2))); 
 }
 static void pl3_file_browser_unselected(GtkWidget *container)
 {
@@ -740,7 +749,6 @@ static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
     {
         GtkTreeSelection *selec = gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_dir_tree));
         GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store);
-        GtkTreeIter iter;
 
         if(gtk_tree_selection_get_selected(selec,&model, &iter))
         {
@@ -772,7 +780,6 @@ static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
     {
         GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)pl3_fb_dir_tree);
         GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store);
-        GtkTreeIter iter;
 
         if(gtk_tree_selection_get_selected(selec,&model, &iter))
         {
@@ -837,23 +844,6 @@ static void pl3_file_browser_add_to_playlist(GtkWidget *menu)
         g_list_free (list);
     }
 }
-static gboolean pl3_file_browser_button_press_event(GtkWidget *but, GdkEventButton *event)
-{
-    GtkTreePath *path = NULL;
-    if(event->button == 3 && gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(but), event->x, event->y,&path,NULL,NULL,NULL))
-    {	
-        GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(but));
-        if(gtk_tree_selection_path_is_selected(sel, path))
-        {
-            gtk_tree_path_free(path);
-            return TRUE;
-        }
-    }
-    if(path) {
-        gtk_tree_path_free(path);
-    }
-    return FALSE;
-}
 static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventButton *event)
 {
 
@@ -869,6 +859,7 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
      */
     if(gtk_tree_selection_count_selected_rows(sel) == 1)
     {
+        mpd_Song *song = NULL;
         GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_store2);
         GList *list = gtk_tree_selection_get_selected_rows(sel, &model);
         if(list != NULL)
@@ -876,6 +867,7 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
             GtkTreeIter iter;
             int row_type;
             char *path;
+            GtkTreePath *tree_path;
             list = g_list_first(list);
             gtk_tree_model_get_iter(model, &iter, list->data);
             gtk_tree_model_get(model, &iter,MPDDATA_MODEL_COL_PATH,&path,MPDDATA_MODEL_ROW_TYPE, &row_type, -1); 
@@ -907,9 +899,6 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
                         G_CALLBACK(pl3_file_browser_update_folder_left_pane), NULL);
                 has_item = 1;
             }
-            g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
-            g_list_free (list);
-            q_free(path);
             if(row_type != -1)
             {
                 /* replace the replace widget */
@@ -928,16 +917,12 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
                 playlist_editor_right_mouse(menu,pl3_file_browser_add_to_playlist);
                 has_item = 1;
             }
-            {
-                mpd_Song *song = NULL;
-                GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_fb_tree));
-                GtkTreePath *path;
-                GtkTreeIter iter;
-                GList *list = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_tree)),&model);
-                path = list->data;
-                /* free result */
-                g_list_free(list);
-                if(path && gtk_tree_model_get_iter(model, &iter, path)) {
+
+///            model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_fb_tree));
+  //          list = gtk_tree_selection_get_selected_rows(sel,&model);
+    //        if(list)
+                tree_path = list->data;
+                if(tree_path && gtk_tree_model_get_iter(model, &iter, tree_path)) {
                     gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
                     if(song)
                     {
@@ -945,9 +930,11 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
                     }
 
                 }
-                if(path)
-                    gtk_tree_path_free(path);
-            }
+            //    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+              //  g_list_free(list);
+            g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
+            g_list_free (list);
+            q_free(path);
         }
     }
     else
@@ -965,6 +952,7 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
         g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_add_selected), NULL);
         has_item = 1;
     }
+    gmpc_mpddata_treeview_right_mouse_intergration(GMPC_MPDDATA_TREEVIEW(pl3_fb_tree), GTK_MENU(menu));
     /*	if(has_item)*/
     {
         item = gtk_image_menu_item_new_with_label(_("Edit Columns"));
@@ -1018,8 +1006,8 @@ static void pl3_file_browser_add_selected()
             {
                 /* add them to the add list */
                 mpd_playlist_queue_add(connection, name);
-                if(type&PL3_ENTRY_DIRECTORY) dirs++;
-                if(type&PL3_ENTRY_SONG) songs++;
+                if(type==MPD_DATA_TYPE_DIRECTORY) dirs++;
+                if(type==MPD_DATA_TYPE_SONG) songs++;
             }
             else if (type == MPD_DATA_TYPE_PLAYLIST)
             {
@@ -1204,7 +1192,12 @@ static int pl3_file_browser_add_go_menu(GtkWidget *menu)
     item = gtk_image_menu_item_new_with_label(_("File Browser"));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
             gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_MENU));
-    gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F3, 0, GTK_ACCEL_VISIBLE);
+    gtk_widget_add_accelerator(GTK_WIDGET(item), 
+            "activate",
+            gtk_menu_get_accel_group(GTK_MENU(menu)), 
+            GDK_F2, 0,
+            GTK_ACCEL_VISIBLE);
+
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     g_signal_connect(G_OBJECT(item), "activate", 
             G_CALLBACK(pl3_file_browser_activate), NULL);
