@@ -18,25 +18,20 @@ static GtkTreeRowReference *playlist_editor_browser_ref = NULL;
 static GtkWidget *playlist_editor_browser = NULL;
 static GtkWidget *playlist_editor_song_tree = NULL;
 static void playlist_editor_status_changed(MpdObj *mi, ChangedStatusType what, void *data);
-static void playlist_editor_fill_list(void);
 static int playlist_editor_add_go_menu(GtkWidget *);
 GtkWidget *playlist_editor_icon_view = NULL;
 GmpcMpdDataModelSort *playlist_editor_list_store = NULL;
-
+void playlist_editor_fill_list_real(void);
 static void playlist_editor_save_myself(void);
 
 enum {
 	PL_NAME,
+    PL_MTIME,
 	PL_IMAGE,
 	PL_NUM_COLS
 };
 
 GtkListStore *playlist_editor_store = NULL;
-/**
- * Functions from gmpc 
- */
-int connection_get_port();
-char *connection_get_hostname();
 
 /**
  * Enable/Disable plugin
@@ -55,19 +50,15 @@ static void playlist_editor_set_enabled(int enabled)
  * Browser extention 
  */
 gmpcPlBrowserPlugin playlist_editor_gbp = {
-  playlist_editor_browser_add,   /* Add */
-  playlist_editor_browser_selected,   /* Selected */
-  playlist_editor_browser_unselected,   /* Unselected */
-  playlist_editor_browser_changed,   /* Changed */
-  NULL,
-  playlist_editor_browser_cat_menu,   /* */
-  NULL,   /* cat key press */
-  playlist_editor_add_go_menu, 
-  NULL 
+  .add = playlist_editor_browser_add,   /* Add */
+  .selected = playlist_editor_browser_selected,   /* Selected */
+  .unselected = playlist_editor_browser_unselected,   /* Unselected */
+  .cat_right_mouse_menu = playlist_editor_browser_cat_menu,   /* */
+  .add_go_menu = playlist_editor_add_go_menu
 };
 
 gmpcPlugin playlist_editor_plugin = {
-  .name 					= "Favorites Plugin",
+  .name 					= "Playlist Editor",
   .version					=  {0,15,0},
   .plugin_type 				= GMPC_PLUGIN_PL_BROWSER,
   .init						= playlist_editor_init,
@@ -104,13 +95,13 @@ void playlist_editor_destroy(void)
  */
 void playlist_editor_conn_changed(MpdObj *mi, int connect, void *userdata)
 {
-	playlist_editor_fill_list();
+	playlist_editor_fill_list_real();
 }
 
 
 void playlist_editor_browser_add(GtkWidget *cat_tree)
 {
-	GtkTreeStore *pl3_tree = playlist3_get_category_tree_store();
+	GtkListStore *store= playlist3_get_category_tree_store();
 	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 	gint pos = cfg_get_single_value_as_int_with_default(config, "playlist-plugin","position",6);
@@ -118,9 +109,8 @@ void playlist_editor_browser_add(GtkWidget *cat_tree)
 	/* Check if enabled */
 	if(!cfg_get_single_value_as_int_with_default(config, "playlist-plugin", "enable", TRUE)) return;
 
-//	gtk_tree_store_append(pl3_tree, &iter, NULL);
 	playlist3_insert_browser(&iter, pos);
-	gtk_tree_store_set(pl3_tree, &iter, 
+	gtk_list_store_set(store, &iter, 
 			PL3_CAT_TYPE, playlist_editor_plugin.id,
 			PL3_CAT_TITLE, _("Playlist Editor"),
 			PL3_CAT_INT_ID, "",
@@ -148,7 +138,7 @@ void playlist_editor_browser_add(GtkWidget *cat_tree)
 
 }
 
-static void playlist_editor_browser_playlist_editor_selected(GtkIconView *giv, GtkTreePath *path, gpointer data)
+static void playlist_editor_browser_playlist_editor_selected(GtkIconView *giv, GtkTreePath *path, gpointer userdata)
 {
 	gchar *pl_path = NULL;
 	GtkTreeIter iter;
@@ -157,10 +147,12 @@ static void playlist_editor_browser_playlist_editor_selected(GtkIconView *giv, G
 	{
 		MpdData *data ;
 		gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), &iter, PL_NAME, &pl_path, -1);
-		data = mpd_database_get_playlist_content(connection, pl_path); 
-		gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), data);		
-		gmpc_mpddata_model_sort_set_playlist(GMPC_MPDDATA_MODEL_SORT(playlist_editor_list_store), pl_path);
-		g_free(pl_path);
+        if(pl_path){
+            data = mpd_database_get_playlist_content(connection, pl_path); 
+            gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), data);		
+            gmpc_mpddata_model_sort_set_playlist(GMPC_MPDDATA_MODEL_SORT(playlist_editor_list_store), pl_path);
+            g_free(pl_path);
+        }
 	}
 }
 static void playlist_editor_browser_playlist_editor_changed(GtkWidget *giv, gpointer data)
@@ -172,40 +164,154 @@ static void playlist_editor_browser_playlist_editor_changed(GtkWidget *giv, gpoi
 	gtk_icon_view_selected_foreach(GTK_ICON_VIEW(giv), playlist_editor_browser_playlist_editor_selected, NULL);
 }
 
-static void playlist_editor_fill_list(void)
+static gint __sort_func(gpointer aa, gpointer bb, gpointer c)
 {
-	if(playlist_editor_browser)
+    MpdData_real *a = *(MpdData_real **)aa;
+    MpdData_real *b = *(MpdData_real **)bb;
+    if(a->type == MPD_DATA_TYPE_PLAYLIST && b->type == MPD_DATA_TYPE_PLAYLIST)
+    {
+        if(a->playlist->path == NULL && b->playlist->path != NULL)
+            return -1;
+        else if(b->playlist->path == NULL && a->playlist->path != NULL)
+            return 1;
+        else if (a->playlist->path  && b->playlist->path)
+        {
+            int val;
+            gchar *sa,*sb;
+            sa = g_utf8_strdown(a->playlist->path, -1);
+            sb = g_utf8_strdown(b->playlist->path, -1);
+            val = g_utf8_collate(sa,sb);
+            g_free(sa);
+            g_free(sb);
+            return val;
+        }
+    }
+    return a->type - b->type;
+}
+void playlist_editor_fill_list(void)
+{
+    if(!mpd_server_has_idle(connection))
+    {
+        playlist_editor_fill_list_real();
+    }
+}
+void playlist_editor_fill_list_real(void)
+{
+    GtkTreeIter iter;
+    if(playlist_editor_browser)
 	{
-		int selected = 0;
-		MpdData *data = mpd_database_get_directory(connection, "/");
-		gtk_list_store_clear(playlist_editor_store);
+        gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playlist_editor_store), &iter);
+//        int selected = 0;
+		MpdData *data = mpd_database_playlist_list(connection);
+        data = misc_sort_mpddata(data, (GCompareDataFunc) __sort_func, NULL);
+//		gtk_list_store_clear(playlist_editor_store);
 		for(;data;data =mpd_data_get_next(data))
 		{
 			if(data->type ==  MPD_DATA_TYPE_PLAYLIST)
 			{
-				GtkTreeIter iter;
-				GdkPixbuf *pb = NULL; 
-				pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
-				gtk_list_store_append(playlist_editor_store, &iter);
-				gtk_list_store_set(playlist_editor_store, &iter,PL_NAME, data->playlist, PL_IMAGE, pb, -1);
-				if(pb)
-					g_object_unref(pb);
-				if(!selected)
-				{
-					GtkTreePath *path;
-					path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist_editor_store), &iter);
-					if(path)
-					{
-						gtk_icon_view_select_path(GTK_ICON_VIEW(playlist_editor_icon_view), path);
-						gtk_tree_path_free(path);
-						selected = TRUE;
-					}
-				}
+loop:            
+                if(valid) {
+                    char *name = NULL;
+                    int val = 0;
+                    gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), &iter, PL_NAME, &name, -1);
+                    val = strcmp(name,data->playlist->path);
+
+                    if(val == 0) { 
+                        GtkTreePath *path = NULL;
+                        gchar *mtime = NULL;
+                        gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), &iter, PL_MTIME, &mtime, -1);
+                        gtk_list_store_set(playlist_editor_store, &iter, PL_MTIME, data->playlist->mtime, -1);
+                        /* do nothing */
+                        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist_editor_store), &iter);
+                        valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(playlist_editor_store), &iter);
+                        if(path) {
+                            if(gtk_icon_view_path_is_selected(GTK_ICON_VIEW(playlist_editor_icon_view), path))
+                            {
+                                if(mtime == NULL || data->playlist->mtime == NULL || strcmp(mtime, data->playlist->mtime) == 0) {
+                                    /* update view */
+                                    playlist_editor_browser_playlist_editor_changed(playlist_editor_icon_view, NULL);
+                                }
+                            }
+                            gtk_tree_path_free(path);
+                        }
+                        g_free(mtime);
+                    }
+                    else if (val > 0) {
+                        GtkTreeIter niter;
+                        GdkPixbuf *pb = NULL; 
+                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
+                        gtk_list_store_insert_before(playlist_editor_store,&niter, &iter);
+                        gtk_list_store_set(playlist_editor_store, &niter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
+                        if(pb)
+                            g_object_unref(pb);
+                    }else{
+                    /*
+                        GtkTreeIter niter;
+                        GdkPixbuf *pb = NULL; 
+                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
+                        gtk_list_store_insert_after(playlist_editor_store,&niter, &iter);
+                        gtk_list_store_set(playlist_editor_store, &niter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
+                        if(pb)
+                            g_object_unref(pb);
+                        iter = niter;
+                        */
+                        valid = gtk_list_store_remove(playlist_editor_store, &iter);
+                        g_free(name);
+                        goto loop;
+                    }
+                    g_free(name);
+
+                }
+                else
+                {
+                    GdkPixbuf *pb = NULL; 
+                    pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
+                    gtk_list_store_append(playlist_editor_store, &iter);
+                    gtk_list_store_set(playlist_editor_store, &iter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
+                    if(pb)
+                        g_object_unref(pb);
+                    
+                        /*
+                    if(!selected)
+                    {
+                        GtkTreePath *path;
+                        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist_editor_store), &iter);
+                        if(path)
+                        {
+                            gtk_icon_view_select_path(GTK_ICON_VIEW(playlist_editor_icon_view), path);
+                            gtk_tree_path_free(path);
+                            selected = TRUE;
+                        }
+                    }
+                    */
+                }
+                
 			}
 		}
+        while(valid) {
+            valid = gtk_list_store_remove(playlist_editor_store, &iter);
+        }
+        if(playlist_editor_browser_ref) {
+            GtkTreeIter piter;
+            GtkTreePath *path;
+            path = gtk_tree_row_reference_get_path(playlist_editor_browser_ref);
+            if(path)
+            {
+                if(gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_tree_row_reference_get_model(playlist_editor_browser_ref)), &piter,path))
+                {
+                    gchar *title = g_strdup_printf("<span color='grey'>(%i)</span>", 
+                            gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playlist_editor_store),NULL));
+                    gtk_list_store_set(GTK_LIST_STORE(gtk_tree_row_reference_get_model(playlist_editor_browser_ref)), &piter,
+                            PL3_CAT_NUM_ITEMS, title, -1);
+                    g_free(title);
+                }
+                gtk_tree_path_free(path);
+            }
+        }
+
 	}
 }
-static void playlist_editor_browser_activate_cursor_item(GtkIconView *giv, gpointer data)
+static void playlist_editor_browser_activate_cursor_item(GtkIconView *giv, gpointer userdata)
 {
 	gchar *pl_path = NULL;
 	GtkTreeIter iter;
@@ -245,10 +351,11 @@ static void playlist_editor_list_delete_songs(GtkButton *button, GtkTreeView *tr
 
 	if(pl_path)
 	{
-		MpdData *data2 = NULL;
-		GtkTreeSelection *select = gtk_tree_view_get_selection(tree);
+//		MpdData *data2 = NULL;
+		GtkTreeSelection *sel = gtk_tree_view_get_selection(tree);
 		GtkTreeModel *model = gtk_tree_view_get_model(tree);
-		GList *data,*list= gtk_tree_selection_get_selected_rows(select, &model);
+		GList *data;
+		list= gtk_tree_selection_get_selected_rows(sel, &model);
 		for(data = g_list_last(list); data; data = g_list_previous(data))
 		{
 			GtkTreePath *path = data->data;
@@ -257,18 +364,17 @@ static void playlist_editor_list_delete_songs(GtkButton *button, GtkTreeView *tr
 			{
 				int *pos = gtk_tree_path_get_indices(path);
 				mpd_database_playlist_list_delete(connection, pl_path,pos[0]);
-
 			}
 		}
-		mpd_playlist_queue_commit(connection);
+        debug_printf(DEBUG_INFO,"ping\n");
 		g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
 		g_list_free (list);
 
-		
+	/*	
 		data2 = mpd_database_get_playlist_content(connection, pl_path); 
 		gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), data2);		
 		gmpc_mpddata_model_sort_set_playlist(GMPC_MPDDATA_MODEL_SORT(playlist_editor_list_store), pl_path);
-
+*/
 		g_free(pl_path);
 	}
 }
@@ -276,9 +382,9 @@ static void playlist_editor_list_delete_songs(GtkButton *button, GtkTreeView *tr
 
 static void playlist_editor_list_add_songs(GtkButton *button, GtkTreeView *tree) 
 {
-	GtkTreeSelection *select = gtk_tree_view_get_selection(tree);
+	GtkTreeSelection *sel= gtk_tree_view_get_selection(tree);
 	GtkTreeModel *model = gtk_tree_view_get_model(tree);
-	GList *data,*list= gtk_tree_selection_get_selected_rows(select, &model);
+	GList *data,*list= gtk_tree_selection_get_selected_rows(sel, &model);
 	for(data = g_list_first(list); data; data = g_list_next(data))
 	{
 		GtkTreePath *path = data->data;
@@ -364,9 +470,10 @@ static void playlist_editor_new_playlist(GtkWidget *item, gpointer data)
 		switch(gtk_dialog_run(GTK_DIALOG(dialog)))
 		{
 			case GTK_RESPONSE_ACCEPT:
-        g_object_set_data_full(G_OBJECT(item), "playlist", g_strdup(gtk_entry_get_text(GTK_ENTRY(entry))), g_free);
-				mpd_database_playlist_clear(connection, gtk_entry_get_text(GTK_ENTRY(entry)));
+                g_object_set_data_full(G_OBJECT(item), "playlist", g_strdup(gtk_entry_get_text(GTK_ENTRY(entry))), g_free);
+                mpd_database_playlist_clear(connection, gtk_entry_get_text(GTK_ENTRY(entry)));
 				playlist_editor_fill_list();
+
 			default:
 				done = TRUE;
 				break;
@@ -422,12 +529,12 @@ static gboolean playlist_editor_key_released(GtkTreeView *tree, GdkEventButton *
 {
 	if(button->button == 3)
 	{
-		GtkTreeSelection *select = gtk_tree_view_get_selection(tree);
+		GtkTreeSelection *sel= gtk_tree_view_get_selection(tree);
 
 
 			GtkWidget *menu = gtk_menu_new();
       GtkWidget *item = NULL;
-      if(gtk_tree_selection_count_selected_rows(select) > 0)
+      if(gtk_tree_selection_count_selected_rows(sel) > 0)
       {
         item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -542,6 +649,14 @@ static gboolean playlist_editor_browser_button_release_event(GtkWidget *giv, Gdk
   }
   return FALSE;
 }
+static void playtime_changed(GmpcMpdDataModel *model, gulong playtime)
+{
+    if(pl3_cat_get_selected_browser() == playlist_editor_plugin.id)
+    {
+        playlist3_show_playtime(playtime);
+    }
+}
+
 
 static void playlist_editor_browser_init()
 {
@@ -557,7 +672,7 @@ static void playlist_editor_browser_init()
     gtk_paned_add1(GTK_PANED(playlist_editor_browser), sw);
 
     /* icon view*/
-    playlist_editor_store = gtk_list_store_new(PL_NUM_COLS, G_TYPE_STRING, GDK_TYPE_PIXBUF);
+    playlist_editor_store = gtk_list_store_new(PL_NUM_COLS, G_TYPE_STRING, G_TYPE_STRING,GDK_TYPE_PIXBUF);
 
     playlist_editor_icon_view = tree = gtk_icon_view_new_with_model(GTK_TREE_MODEL(playlist_editor_store));
     gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(tree), GTK_SELECTION_BROWSE);
@@ -580,6 +695,9 @@ static void playlist_editor_browser_init()
 
 
     playlist_editor_list_store= gmpc_mpddata_model_sort_new();
+    gmpc_mpddata_model_disable_image(GMPC_MPDDATA_MODEL(playlist_editor_list_store));
+    g_signal_connect(G_OBJECT(playlist_editor_list_store), "playtime_changed", G_CALLBACK(playtime_changed), NULL);
+
 
     playlist_editor_song_tree = tree = gmpc_mpddata_treeview_new("playlist-browser",FALSE, GTK_TREE_MODEL(playlist_editor_list_store));
     gtk_container_add(GTK_CONTAINER(sw), tree);
@@ -596,10 +714,12 @@ void playlist_editor_browser_selected(GtkWidget *container)
 {
     if(!playlist_editor_browser) {
         playlist_editor_browser_init();
-        playlist_editor_fill_list();
+        playlist_editor_fill_list_real();
     }
     gtk_container_add(GTK_CONTAINER(container), playlist_editor_browser);
     gtk_widget_show_all(playlist_editor_browser);
+
+    playlist3_show_playtime(gmpc_mpddata_model_get_playtime(GMPC_MPDDATA_MODEL(playlist_editor_list_store))); 
 }
 
 void playlist_editor_browser_unselected(GtkWidget *container)
@@ -607,12 +727,6 @@ void playlist_editor_browser_unselected(GtkWidget *container)
     gtk_container_remove(GTK_CONTAINER(container), playlist_editor_browser);
 }
 
-void playlist_editor_browser_changed(GtkWidget *tree, GtkTreeIter *iter)
-{
-    playlist_editor_fill_list();
-}
-
-
 int playlist_editor_browser_cat_menu(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event)
 {
     if(type == playlist_editor_plugin.id)
@@ -650,15 +764,15 @@ void playlist_editor_right_mouse(GtkWidget *menu, void (*add_to_playlist)(GtkWid
     gtk_menu_shell_append(GTK_MENU_SHELL(smenu), sitem);
 
     {
-        MpdData *data = mpd_database_get_directory(connection, "/");
+        MpdData *data = mpd_database_playlist_list(connection);
         for(;data;data =mpd_data_get_next(data))
         {
             if(data->type ==  MPD_DATA_TYPE_PLAYLIST)
             {
-                sitem = gtk_image_menu_item_new_with_label(data->playlist);
+                sitem = gtk_image_menu_item_new_with_label(data->playlist->path);
                 gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), 
                         gtk_image_new_from_icon_name("media-playlist", GTK_ICON_SIZE_MENU));
-                g_object_set_data_full(G_OBJECT(sitem),"playlist", g_strdup(data->playlist), g_free);
+                g_object_set_data_full(G_OBJECT(sitem),"playlist", g_strdup(data->playlist->path), g_free);
                 gtk_menu_shell_append(GTK_MENU_SHELL(smenu), sitem);
                 g_signal_connect(G_OBJECT(sitem), "activate", G_CALLBACK(add_to_playlist), NULL);
                 gtk_widget_show(sitem);                             				
@@ -677,10 +791,10 @@ void playlist_editor_right_mouse(GtkWidget *menu, void (*add_to_playlist)(GtkWid
 
 static void playlist_editor_status_changed(MpdObj *mi, ChangedStatusType what, void *data)
 {
-    /*	if(what&MPD_CST_STORED_PLAYLIST)
-        {
-        playlist_editor_fill_list();
-        }*/
+    if(what&MPD_CST_STORED_PLAYLIST)
+    {
+        playlist_editor_fill_list_real();
+    }
 }
 static void playlist_editor_activate(GtkWidget *item, gpointer data)
 {
@@ -707,7 +821,11 @@ static int playlist_editor_add_go_menu(GtkWidget *menu)
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
             gtk_image_new_from_icon_name("media-playlist", GTK_ICON_SIZE_MENU));
 
-    gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F7, 0, GTK_ACCEL_VISIBLE);
+    gtk_widget_add_accelerator(GTK_WIDGET(item), 
+            "activate", 
+            gtk_menu_get_accel_group(GTK_MENU(menu)), 
+            GDK_F5, 0, 
+            GTK_ACCEL_VISIBLE);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     g_signal_connect(G_OBJECT(item), "activate", 
             G_CALLBACK(playlist_editor_activate), NULL);
