@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
+ *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -60,29 +60,36 @@ typedef struct _config_obj
 	int total_size;
 } _config_obj;
 
-static void __int_cfg_set_single_value_as_string(config_obj *, char *, char *, char *);
-static void cfg_save(config_obj *);
+static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, char *value);
+static void cfg_save_real(config_obj *);
 static void __int_cfg_remove_node(config_obj *, config_node *);
-
+static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, char *class);
 static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node);
-static config_node *cfg_add_class(config_obj *, char *);
+static config_node *cfg_add_class(config_obj *, config_node *,char *);
 static config_node *cfg_new_node(void);
 static void cfg_add_child(config_node *, config_node *);
+#define cfg_save(a) debug_printf(DEBUG_INFO, "Save triggered");cfg_save_real(a)
 
 static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 {
 	char buffer[1024];
 	int len = 0;
-	int c;
+	int c,version = 0;
 	config_node *cur = NULL;
 	config_node *multiple = NULL;
+    int firstchar = 0;
 	while((c = fgetc(fp)) != EOF)
 	{
-		if(c == '[')
+        if(c == '2' && !firstchar)
+        {
+            version = 2;
+        }
+        firstchar = 1; 
+        if(c == '[')
 		{
 			len =0;
 			c = fgetc(fp);
-			while(c != ']' && c != EOF && len < 1024){
+			while(c != ']' && c != EOF && len < 1023){
 				buffer[len] = c;
 				len++;
 				c = fgetc(fp);
@@ -90,8 +97,13 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 			buffer[len] = '\0';
 			if(len > 0 && len < 256)
 			{
-				cur = cfg_add_class(cfgo, buffer);
+//                printf("add class: %s -> %s\n", buffer, (cur )?cur->name:"");
+				cur = cfg_add_class(cfgo, (version <2)?NULL:cur, buffer);
 			}
+            else if (len == 0) {
+                if(version < 2) { cur = NULL; }
+                else if(cur){ cur = cur->parent; }
+            }
 			/* seek end of line */
 			while(c != EOF && c != '\n') c = fgetc(fp);
 		}
@@ -99,7 +111,7 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 		{
 			len =0;                                    		
 			c = fgetc(fp);
-			while(c != '}' && c != EOF && len < 1024){
+			while(c != '}' && c != EOF && len < 1023){
 				buffer[len] = c;
 				len++;
 				c = fgetc(fp);
@@ -110,6 +122,7 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 				config_node *child = cfg_new_node();
 				child->type = TYPE_ITEM_MULTIPLE;
 				child->name = g_strndup(buffer, len);
+                child->parent = cur;
 				child->children = NULL;
 				cfg_add_child(cur, child);
 				multiple = child;
@@ -142,6 +155,7 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 
 				/* write key name */
 				new = cfg_new_node();
+                new->parent = cur;
 				new->type = TYPE_ITEM;
 				new->name = g_strndup(buffer, len);	
 				cfgo->total_size+= len+sizeof(config_node);
@@ -177,7 +191,7 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 						len++;
 					}
 					c = fgetc(fp);
-				}while((c != '\n' || quote) && c != EOF && quote >= 0 && len < 1024);
+				}while((c != '\n' || quote) && c != EOF && quote >= 0 && len < 1023);
 				new->value = g_strndup(buffer, len);
 				cfgo->total_size+= len;
 				if(multiple){
@@ -190,6 +204,7 @@ static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 			while(c != EOF && c != '\n') c = fgetc(fp);			
 		}
 		else while(c != EOF && c != '\n') c = fgetc(fp);			
+    
 	}
 }
 
@@ -235,6 +250,7 @@ void cfg_close(config_obj *cfgo)
 	{
 		return;
 	}
+    cfg_save(cfgo);
 	debug_printf(DEBUG_INFO,"Closing config '%s' with %i bytes allocated\n", cfgo->url, cfgo->total_size);
 	g_mutex_lock(cfgo->lock);
 	if(cfgo->url != NULL)
@@ -261,25 +277,42 @@ static config_node *cfg_new_node()
 	newnode->value = NULL;
 	return newnode;
 }
-static config_node *cfg_add_class(config_obj *cfg, char *class)
+static config_node *cfg_add_class(config_obj *cfg,config_node *parent, char *class)
 {
 	config_node *newnode = cfg_new_node();
 	newnode->type = TYPE_CATEGORY;
 	newnode->name = g_strdup(class);
 	newnode->value = NULL;
+    newnode->parent = parent;
 	newnode->children  = NULL;
 	cfg->total_size += sizeof(config_node)+strlen(class);
-	if(cfg->root == NULL)
-	{
-		cfg->root = newnode;		
-	}
-	else
-	{
-		config_node *temp = cfg->root;
-		while(temp->next != NULL) temp = temp->next;
-		temp->next = newnode;
-		newnode->prev = temp;
-	}
+    if(!parent)
+    {
+        if(cfg->root == NULL)
+        {
+            cfg->root = newnode;		
+        }
+        else
+        {
+            config_node *temp = cfg->root;
+            while(temp->next != NULL) temp = temp->next;
+            temp->next = newnode;
+            newnode->prev = temp;
+        }
+    }
+    else 
+    {
+        if(parent->children == NULL) {
+            parent->children = newnode;
+        }else{
+            config_node *temp = parent->children;              
+            /* get last node */
+            while(temp->next != NULL) temp = temp->next;
+            temp->next = newnode;
+            newnode->prev = temp;	
+        }
+    }
+
 	return newnode;
 }
 void cfg_add_child(config_node *parent, config_node *child)
@@ -306,6 +339,7 @@ void cfg_add_child(config_node *parent, config_node *child)
 static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 {
 	config_node *temp = NULL;
+
 	if(node == NULL)return;
 	/* find the first */
 	while(node->prev != NULL) node = node->prev;
@@ -313,20 +347,27 @@ static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 	for(temp = node;temp != NULL; temp = temp->next){
 		if(temp->type == TYPE_CATEGORY)
 		{
-			fprintf(fp, "\n[%s]\n\n",temp->name);
+            fputs("\n[", fp);
+            fputs(temp->name, fp);
+            fputs("]\n",fp);
 			cfg_save_category(cfg,temp->children,fp);
+            fputs("[]\n",fp);
 		}
 		if(temp->type == TYPE_ITEM_MULTIPLE)
 		{
-			fprintf(fp, "\n{%s}\n",temp->name);
+            fputs("\n{", fp);
+            fputs(temp->name, fp);
+            fputs("}\n",fp);
 			cfg_save_category(cfg,temp->children,fp);
-			fprintf(fp, "{}\n\n");
-		}                                                		
+            fputs("{}\n\n",fp);
+        }                                                		
 		else if (temp->type == TYPE_ITEM)
 		{
 			int i= 0;
-			fprintf(fp, "%s=\"", temp->name);
-			for(i=0;i<strlen(temp->value);i++)
+            int length = strlen(temp->value);
+            fputs(temp->name, fp);
+            fputs("=\"", fp);
+			for(i=0;i<length;i++)
 			{
 				if(temp->value[i] == '"'){
 					fputs("\\\"",fp);
@@ -338,7 +379,7 @@ static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 					fputs("\\n",fp);
 				}
 				else{
-					fputc(temp->value[i],fp);
+					putc(temp->value[i],fp);
 				}
 			}
 			fputs("\"\n",fp);
@@ -346,16 +387,18 @@ static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 	}
 }
 
-static void cfg_save(config_obj *cfgo)
+static void cfg_save_real(config_obj *cfgo)
 {
 	if(cfgo == NULL)
 	{
 		return;
 	}
+    debug_printf(DEBUG_INFO,"Saving config file: %s (%i bytes)", cfgo->url, cfgo->total_size);
 	if(cfgo->root != NULL)
 	{
 		FILE *fp = fopen(cfgo->url, "w");
 		if(!fp) return;
+        fputs("2\n", fp);
 		cfg_save_category(cfgo,cfgo->root, fp);	
 		fclose(fp);
 #ifndef WIN32
@@ -368,7 +411,19 @@ static void cfg_save(config_obj *cfgo)
 
 static config_node *cfg_get_class(config_obj *cfg, char *class)
 {
-	config_node *node = cfg->root;
+    return cfg_get_class_multiple(cfg, NULL, class);
+}
+
+static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, char *class)
+{
+	config_node *node = NULL;
+    if(root != NULL)
+    {
+        node = root->children;
+
+    }else{
+        node = cfg->root;
+    }
 	if(node == NULL) return NULL;
 	/* find the first */
 	while(node->prev != NULL) node = node->prev;
@@ -384,7 +439,7 @@ static config_node *cfg_get_class(config_obj *cfg, char *class)
 	return NULL;
 }
 
-static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key)
+static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
 {
 	/* take children */
 	config_node *cur = NULL;
@@ -393,7 +448,15 @@ static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key
 	{
 		return NULL;
 	}
-	cur = cur->children;
+    if(sclass) {
+        cur = cfg_get_class_multiple(cfg, cur, sclass);
+        if(cur == NULL || cur->children == NULL) return NULL;
+    }
+    if(ssclass) {
+        cur = cfg_get_class_multiple(cfg, cur, ssclass);
+        if(cur == NULL || cur->children == NULL) return NULL;
+    }
+    cur = cur->children;
 	for(;cur != NULL; cur = cur->next) {
 		if(!strcmp(cur->name, key))
 		{
@@ -411,9 +474,9 @@ static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *key
 	}
 }
 */
-static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
+static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class, char * sclass, char *ssclass, char *key)
 {
-	config_node *cur = cfg_get_single_value(cfg, class,key);
+	config_node *cur = cfg_get_single_value(cfg, class, sclass, ssclass,key);
 	if(cur != NULL)
 	{
 		if(cur->type == TYPE_ITEM)
@@ -426,34 +489,21 @@ static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class,
 
 char * cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
 {
-	char *retv = NULL;
-	g_mutex_lock(cfg->lock);
-	retv = __int_cfg_get_single_value_as_string(cfg,class,key);
-	g_mutex_unlock(cfg->lock);
-	return retv;
+    return cfg_get_single_value_as_string_mm(cfg, class, NULL, NULL,key);
 }
 
 
 char * cfg_get_single_value_as_string_with_default(config_obj *cfg, char *class, char *key , char *def)
 {
-	char *retv = NULL;
-   	g_mutex_lock(cfg->lock);
-	retv = __int_cfg_get_single_value_as_string(cfg,class,key);
-	if(retv == NULL)
-	{
-		__int_cfg_set_single_value_as_string(cfg,class,key,def);
-		retv = __int_cfg_get_single_value_as_string(cfg,class,key);
-	}
-	g_mutex_unlock(cfg->lock);
-	return retv;
+    return cfg_get_single_value_as_string_with_default_mm(cfg, class, NULL, NULL,key,def);
 }
 
-static int __int_cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
+static int __int_cfg_get_single_value_as_int_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
 {
 	config_node *cur =NULL;
 	int retv = CFG_INT_NOT_DEFINED;
 
-	cur = cfg_get_single_value(cfg, class,key);
+	cur = cfg_get_single_value(cfg, class,sclass, ssclass,key);
 	if(cur != NULL)
 	{
 		if(cur->type == TYPE_ITEM)
@@ -468,46 +518,33 @@ int cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
 {
 	int retv = 0;
 	g_mutex_lock(cfg->lock);
-	retv = __int_cfg_get_single_value_as_int(cfg, class, key);
+	retv = __int_cfg_get_single_value_as_int_mm(cfg, class,NULL, NULL, key);
 	g_mutex_unlock(cfg->lock);
 	return retv;
 }
-
 void cfg_set_single_value_as_int(config_obj *cfg, char *class, char *key, int value)
 {
-	gchar *	temp = NULL;
-	g_mutex_lock(cfg->lock);
-	temp  = g_strdup_printf("%i",value);
-	__int_cfg_set_single_value_as_string(cfg,class,key,temp);
-	cfg_free_string(temp);
-	g_mutex_unlock(cfg->lock);
+    cfg_set_single_value_as_int_mm(cfg,class,NULL, NULL, key,value);
 }
-static void __int_cfg_set_single_value_as_int(config_obj *cfg, char *class, char *key, int value)
+static void __int_cfg_set_single_value_as_int(config_obj *cfg, char *class, char *sclass, char *ssclass,char *key, int value)
 {
 	gchar *	temp = NULL;
 	temp  = g_strdup_printf("%i",value);
-	__int_cfg_set_single_value_as_string(cfg,class,key,temp);
+	__int_cfg_set_single_value_as_string(cfg,class,sclass, ssclass,key,temp);
 	cfg_free_string(temp);
 }
+
 int cfg_get_single_value_as_int_with_default(config_obj *cfg, char *class, char *key, int def)
 {
-	int retv = CFG_INT_NOT_DEFINED;
-	g_mutex_lock(cfg->lock);		
-	retv = __int_cfg_get_single_value_as_int(cfg,class,key);
-	if(retv == CFG_INT_NOT_DEFINED)
-	{
-		__int_cfg_set_single_value_as_int(cfg,class,key,def);
-		retv = __int_cfg_get_single_value_as_int(cfg,class,key);		
-	}
-	g_mutex_unlock(cfg->lock);		
-	return retv;
+    return  cfg_get_single_value_as_int_with_default_mm(cfg, class,NULL, NULL, key, def);
 }
 /* float */
-static float __int_cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
+static float __int_cfg_get_single_value_as_float(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
 {
-	config_node *cur =NULL;
-	cur = cfg_get_single_value(cfg,class,key);
 	float result = 0;
+	config_node *cur =NULL;
+	cur = cfg_get_single_value(cfg,class,sclass, ssclass, key);
+	
 	if(cur == NULL)
 	{
 		return CFG_INT_NOT_DEFINED;
@@ -521,7 +558,17 @@ float cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
 {
 	float retv = 0;
 	g_mutex_lock(cfg->lock);
-	retv = __int_cfg_get_single_value_as_float(cfg,class,key);
+	retv = __int_cfg_get_single_value_as_float(cfg,class,NULL, NULL,key);
+	g_mutex_unlock(cfg->lock);
+	return retv;
+}
+
+
+float cfg_get_single_value_as_float_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
+{
+	float retv = 0;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_float(cfg,class,sclass,ssclass,key);
 	g_mutex_unlock(cfg->lock);
 	return retv;
 }
@@ -531,7 +578,7 @@ void cfg_set_single_value_as_float(config_obj *cfg, char *class, char *key, floa
 	char *value1 = NULL;
 	g_mutex_lock(cfg->lock);
 	value1 = g_strdup_printf("%f", value);
-	__int_cfg_set_single_value_as_string(cfg,class,key,value1);
+	__int_cfg_set_single_value_as_string(cfg,class,NULL, NULL, key,value1);
 	cfg_free_string(value1);
 	g_mutex_unlock(cfg->lock);
 }
@@ -539,13 +586,13 @@ float cfg_get_single_value_as_float_with_default(config_obj *cfg, char *class, c
 {
 	float retv = 0;
 	g_mutex_lock(cfg->lock);
-	retv = __int_cfg_get_single_value_as_float(cfg,class,key);
+	retv = __int_cfg_get_single_value_as_float(cfg,class,NULL, NULL, key);
 	if(retv == CFG_INT_NOT_DEFINED)
 	{
 		char * value1 = g_strdup_printf("%f", def);
-		__int_cfg_set_single_value_as_string(cfg,class,key,value1);
+		__int_cfg_set_single_value_as_string(cfg,class,NULL, NULL, key,value1);
 		cfg_free_string(value1);
-		retv = __int_cfg_get_single_value_as_float(cfg,class,key);		
+		retv = __int_cfg_get_single_value_as_float(cfg,class,NULL, NULL, key);		
 	}
 	g_mutex_unlock(cfg->lock);
 	/* make it return an error */
@@ -608,18 +655,23 @@ static void __int_cfg_remove_node(config_obj *cfg, config_node *node)
 	}
 	g_slice_free(config_node,node);
 }
-void cfg_del_single_value(config_obj *cfg, char *class, char *key)
+void cfg_del_single_value_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
 {
 	config_node *node = NULL;
 	g_mutex_lock(cfg->lock);
-   	node = cfg_get_single_value(cfg,class,key);
+   	node = cfg_get_single_value(cfg,class,sclass,ssclass, key);
 	if(node != NULL)
 	{
 		__int_cfg_remove_node(cfg,node);
-		cfg_save(cfg);
+		//cfg_save(cfg);
 	}
 	g_mutex_unlock(cfg->lock);
 }
+void cfg_del_single_value(config_obj *cfg, char *class, char *key)
+{
+    cfg_del_single_value_mm(cfg,class,NULL, NULL, key);
+}
+
 void cfg_remove_class(config_obj *cfg, char *class)
 {
 	config_node *node = NULL;
@@ -632,20 +684,38 @@ void cfg_remove_class(config_obj *cfg, char *class)
 	{
 		__int_cfg_remove_node(cfg, node);
 	}
-	cfg_save(cfg);
+	//cfg_save(cfg);
 	g_mutex_unlock(cfg->lock);
 }
-static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
+static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, char *value)
 {
-	config_node *newnode = cfg_get_single_value(cfg,class,key);
+	config_node *newnode = cfg_get_single_value(cfg,class,sclass, ssclass, key);
 	if(newnode == NULL)
 	{	
 		config_node *node = cfg_get_class(cfg, class);
 		if(node == NULL)
 		{
-			node = cfg_add_class(cfg, class);	
+			node = cfg_add_class(cfg,NULL, class);	
 			if(node == NULL) return;
 		}
+        if(sclass) {
+            config_node *nnode = cfg_get_class_multiple(cfg, node, sclass);
+            if(nnode == NULL)
+            {
+                nnode = cfg_add_class(cfg,node, sclass);	
+                if(nnode == NULL) return;
+            }
+            node = nnode;
+        }
+        if(ssclass) {
+            config_node *nnode = cfg_get_class_multiple(cfg, node, ssclass);
+            if(nnode == NULL)
+            {
+                nnode = cfg_add_class(cfg,node, ssclass);	
+                if(nnode == NULL) return;
+            }
+            node = nnode;
+        }
 		newnode = cfg_new_node();
 		newnode->name = g_strdup(key);
 		cfg->total_size+=sizeof(config_node)+strlen(key);
@@ -664,13 +734,13 @@ static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, c
 	}
 	newnode->value = g_strdup(value);	
 	cfg->total_size += strlen(value);
-	cfg_save(cfg);
+	//cfg_save(cfg);
 }
 
 void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
 {
 	g_mutex_lock(cfg->lock);
-	__int_cfg_set_single_value_as_string(cfg, class,key,value);
+	__int_cfg_set_single_value_as_string(cfg, class,NULL, NULL, key,value);
 	g_mutex_unlock(cfg->lock);
 }
 
@@ -679,7 +749,7 @@ void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, cha
 
 static config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *key, char* id)
 {
-	config_node *node = cfg_get_single_value(cfg,class,key);
+	config_node *node = cfg_get_single_value(cfg,class,NULL, NULL,key);
 	if(node == NULL){
 		return NULL;
 	}
@@ -710,7 +780,7 @@ void cfg_del_multiple_value(config_obj *cfg, char *class, char *key,char *id)
 	if(cur != NULL)
 	{
 		__int_cfg_remove_node(cfg,cur);
-		cfg_save(cfg);
+		//cfg_save(cfg);
 	}
 	g_mutex_unlock(cfg->lock);
 }
@@ -733,15 +803,15 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 
 		cur->value = g_strdup(value);
 		cfg->total_size += strlen(cur->value);
-		cfg_save(cfg);
+		//cfg_save(cfg);
 	}
 	else {
-		config_node *node = cfg_get_single_value(cfg,class,key);
+		config_node *node = cfg_get_single_value(cfg,class,NULL, NULL,key);
 		if(node == NULL) {
 			node = cfg_get_class(cfg,class);
 			if(node == NULL)
 			{
-				node = cfg_add_class(cfg,class);
+				node = cfg_add_class(cfg,NULL,class);
 			}
 			cur = cfg_new_node();
 			cur->name = g_strdup(key);
@@ -757,7 +827,7 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 		cur->value = g_strdup(value);
 		cfg->total_size+=sizeof(config_node)+strlen(id)+strlen(value);;
 		cfg_add_child(node,cur);
-		cfg_save(cfg);
+		//cfg_save(cfg);
 	}
 	g_mutex_unlock(cfg->lock);
 }
@@ -767,7 +837,7 @@ conf_mult_obj * cfg_get_multiple_as_string(config_obj *cfg, char *class, char *k
 	conf_mult_obj *list = NULL;
 	config_node *cur = NULL;
 	g_mutex_lock(cfg->lock);
-	cur = cfg_get_single_value(cfg, class,key);
+	cur = cfg_get_single_value(cfg, class,NULL, NULL, key);
 	if(cur && cur->type == TYPE_ITEM_MULTIPLE && cur->children != NULL)
 	{
 		cur = cur->children;	
@@ -896,20 +966,24 @@ static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node)
 				__int_cfg_remove_node(cfg, root);	
 				root = item;
 			}
-		}	
+            else
+                root = root->next;
+        }	
 		else if(root->type == TYPE_ITEM)
 		{
 			if(root->value == NULL || root->value[0] == '\0' || strlen(root->value) == 0 )
 			{
-				config_node *node = root;
-				root = node->prev;
+				config_node *inode = root;
+				root = inode->prev;
 				if(!root)
-					root = node->next;
-				__int_cfg_remove_node(cfg, node);
+					root = inode->next;
+				__int_cfg_remove_node(cfg,inode);
 				if(!root) return;
 			}
+            else 
+                root = root->next;
 		}
-        if(root)
+        else if(root)
             root = root->next;
 	}
 }
@@ -923,3 +997,65 @@ void cfg_do_special_cleanup(config_obj *cfg)
 	cfg_save(cfg);
 	g_mutex_unlock(cfg->lock);
 }
+
+/** 
+ * Full api for use with metadata
+ */
+
+/**
+ * String 
+ */
+void cfg_set_single_value_as_string_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key, char *value)
+{
+	g_mutex_lock(cfg->lock);
+	__int_cfg_set_single_value_as_string(cfg, class,sclass,ssclass, key,value);
+	g_mutex_unlock(cfg->lock);
+}
+char * cfg_get_single_value_as_string_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
+{
+	char *retv = NULL;
+	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_string(cfg,class,sclass,ssclass,key);
+	g_mutex_unlock(cfg->lock);
+	return retv;
+}
+
+
+char * cfg_get_single_value_as_string_with_default_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key , char *def)
+{
+	char *retv = NULL;
+   	g_mutex_lock(cfg->lock);
+	retv = __int_cfg_get_single_value_as_string(cfg,class,sclass,ssclass,key);
+	if(retv == NULL)
+	{
+		__int_cfg_set_single_value_as_string(cfg,class,sclass, ssclass, key,def);
+		retv = __int_cfg_get_single_value_as_string(cfg,class,sclass, ssclass,key);
+	}
+	g_mutex_unlock(cfg->lock);
+	return retv;
+}
+/**
+ * Int 
+ */
+void cfg_set_single_value_as_int_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key, int value)
+{
+	gchar *	temp = NULL;
+	g_mutex_lock(cfg->lock);
+	temp  = g_strdup_printf("%i",value);
+	__int_cfg_set_single_value_as_string(cfg,class,sclass, ssclass,key,temp);
+	cfg_free_string(temp);
+	g_mutex_unlock(cfg->lock);
+}
+int cfg_get_single_value_as_int_with_default_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, int def)
+{
+	int retv = CFG_INT_NOT_DEFINED;
+	g_mutex_lock(cfg->lock);		
+	retv = __int_cfg_get_single_value_as_int_mm(cfg,class,sclass, ssclass,key);
+	if(retv == CFG_INT_NOT_DEFINED)
+	{
+		__int_cfg_set_single_value_as_int(cfg,class,sclass, ssclass,key,def);
+		retv = __int_cfg_get_single_value_as_int_mm(cfg,class,sclass, ssclass, key);		
+	}
+	g_mutex_unlock(cfg->lock);		
+	return retv;
+}
