@@ -8,9 +8,12 @@
 #include "main.h"
 #define CURL_TIMEOUT 10 
 
-static size_t write_data(void *buffer, size_t size, size_t nmemb, gmpc_easy_download_struct *dld)
+static int quit = FALSE;
+
+static size_t write_data(void *buffer, size_t size, size_t nmemb,void *stream )
 {
-	if(!size || !nmemb)
+    gmpc_easy_download_struct *dld = stream;
+        if(!size || !nmemb)
 		return 0;
 	if(dld->data == NULL)
 	{
@@ -39,6 +42,7 @@ int gmpc_easy_download(const char *url,gmpc_easy_download_struct *dld)
 	CURLM *curlm = NULL;
 	CURLMsg *msg = NULL;
 	double total_size = 0;
+    if(quit) return 0;
 	/*int res;*/
 	if(!dld) return 0;
 	if(url == NULL) return 0;
@@ -47,6 +51,20 @@ int gmpc_easy_download(const char *url,gmpc_easy_download_struct *dld)
 	 * Make sure it's clean
 	 */
 	gmpc_easy_download_clean(dld);
+
+    /** Check for local url */
+    if(strncmp(url, "http://", 7) && g_file_test(url, G_FILE_TEST_EXISTS))
+    {
+        gsize size;
+        if(g_file_get_contents(url, &(dld->data),&(size), NULL))
+        {
+            dld->size = (int)size;
+            return 1;
+        }
+        return 0;
+    }
+
+
 	/* initialize curl */
 	curl = curl_easy_init();
 	if(!curl) return 0;
@@ -67,6 +85,7 @@ int gmpc_easy_download(const char *url,gmpc_easy_download_struct *dld)
 	curl_easy_setopt(curl, CURLOPT_AUTOREFERER ,1);
 	/* set NO SIGNAL */
 	curl_easy_setopt(curl, CURLOPT_NOSIGNAL, TRUE);
+    curl_easy_setopt(curl,CURLOPT_ENCODING ,"");
 
 	if(cfg_get_single_value_as_int_with_default(config, "Network Settings", "Use Proxy", FALSE))
 	{
@@ -120,7 +139,9 @@ int gmpc_easy_download(const char *url,gmpc_easy_download_struct *dld)
 				}
 			}
 		}
-	}while(running);
+	}while(running && !quit);
+    if(quit)
+        debug_printf(DEBUG_INFO, "Forced quit easy_download");
 	/**
 	 * remove handler
 	 */
@@ -171,7 +192,9 @@ static void proxy_pref_construct(GtkWidget *container)
 	/* Create frame and create a widget with markup for the frame */
 	proxy_pref_frame = gtk_frame_new("");
 	temp = gtk_label_new(_("Proxy"));
-	gtk_label_set_markup(GTK_LABEL(temp), _("<b>Proxy</b>"));
+	value = g_markup_printf_escaped("<b>%s</b>", _("Proxy"));
+	gtk_label_set_markup(GTK_LABEL(temp),value); 
+	g_free(value);
 	gtk_frame_set_label_widget(GTK_FRAME(proxy_pref_frame), temp);
 	gtk_frame_set_shadow_type(GTK_FRAME(proxy_pref_frame), GTK_SHADOW_NONE);
 	/* setup vbox for inside the frame */
@@ -213,6 +236,12 @@ static void proxy_pref_construct(GtkWidget *container)
 	gtk_widget_show_all(proxy_pref_frame);
 	gtk_container_add(GTK_CONTAINER(container), proxy_pref_frame);
 }
+
+void quit_easy_download(void)
+{
+    debug_printf(DEBUG_INFO,"quitting easy download\n");
+    quit = TRUE;
+}
 gmpcPrefPlugin proxyplug_pref = {
 	.construct		= proxy_pref_construct,
 	.destroy		= proxy_pref_destroy
@@ -222,6 +251,5 @@ gmpcPlugin proxyplug = {
 	.version 	 	= {0,0,0},
 	.plugin_type	= GMPC_INTERNALL,
 	.pref			= &proxyplug_pref
-
 };
 
