@@ -14,6 +14,7 @@
 #  endif
 #else 
 #define _(String) String
+#define N_(String) String
 #endif
 
 #include "config-defaults.h"
@@ -25,77 +26,33 @@
 #include "plugin.h"
 #include "metadata.h"
 #include "playlist3.h"
-#include "playlist-list.h"
 #include "mpdinteraction.h"
-#include "gmpc-profiles.h"
+
 #include "playlist3-messages.h" 
-#include "browsers/playlist3-playlist-editor.h"
+
+
 /**
  * Some gobjects
  */
-#include "gmpc-connection.h"
-#include "gmpc-metaimage.h"
-#include "gmpc-signals.h"
+#include "gob/gmpc-profiles.h"
+#include "gob/gmpc-connection.h"
+#include "gob/gmpc-metaimage.h"
+#include "gob/gmpc-signals.h"
+#include "gob/gmpc-idle.h"
 
-extern GmpcConnection *gmpcconn;
-extern GmpcSignals *gmpc_signals;
-
-
-extern int gmpc_connected;
-
-extern GtkTreeModel *playlist;
-
-/* the config object */
-extern config_obj *config;
-extern config_obj *profiles;
+extern GmpcConnection 	*gmpcconn;
+extern GmpcSignals 		*gmpc_signals;
+extern GmpcIdle 		*gmpc_idle;
+extern int 				gmpc_connected;
+extern GtkTreeModel 	*playlist;
 
 /* the plugin list */
 extern gmpcPlugin **plugins;
 /* num of plugins */
 extern int num_plugins;
 
-/* the plugins :D */
-
-
-
-
-extern gmpcPlugin connection_plug;
-/*
-extern gmpcPlugin about_plug;
-*/
-extern gmpcPlugin playlist_plug;
-
-extern gmpcPlugin cover_art_plug;
-
-extern gmpcPlugin tray_icon_plug;
-extern gmpcPlugin tray_icon2_plug;
-extern gmpcPlugin proxyplug;
-//extern gmpcPlugin info3_plugin;
-extern gmpcPlugin metab_plugin;
-extern gmpcPlugin url_plugin; 
-/** main.c **/
-extern GladeXML *xml_main_window;
-extern MpdObj *connection;
-extern GmpcProfiles *gmpc_profiles;
-
-extern gmpcPlugin playlist_editor_plugin;
-extern gmpcPlugin pl3_cat_editor_plug;
-
-enum{
-	TIME_FORMAT_ELAPSED,
-	TIME_FORMAT_REMAINING,
-	TIME_FORMAT_PERCENTAGE
-};
-
-/* main.h*/
-void main_trigger_update();
-
-void pl3_highlight_song_change ();
 char * edit_song_markup(char *format);
 
-gboolean playlist_filter_func(GtkTreeModel *model, GtkTreeIter *iter);
-void playlist_changed(MpdObj *mi);
-
 void main_quit(void);
 
 /*
@@ -111,11 +68,6 @@ int plugin_get_pos(int id);
 
 void show_error_message(gchar *string, int block);
 
-/* This is over here because of it's need for xml_error_window, this should probably
- * be moved to mpdinteraction.c
- */
-void connect_callback(MpdObj *mi);
-
 /** Handle status changed */
 void   GmpcStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
 
@@ -128,6 +80,7 @@ void meta_data_add_plugin(gmpcPlugin *plug);
  * TODO move this 
  */
 void url_start_real(const gchar *url);
+void url_start(void);
 
 /*
  * functions to get patch to different files.
@@ -135,17 +88,15 @@ void url_start_real(const gchar *url);
  */
 char *gmpc_get_full_image_path(char *filename);
 
-void preferences_show_pref_window(int plugin_id);
-
 /**
  * Help functions
  */
-#define q_free(a) g_free(a);a=NULL;
+#define q_free(a) {g_free(a);a=NULL;}
 
 /* help */
 void info2_activate(void);
-void info2_fill_artist_view(char *);
-void info2_fill_album_view(char *, char *);
+void info2_fill_artist_view(const char *);
+void info2_fill_album_view(const char *,const char *);
 void info2_fill_song_view(mpd_Song *);
 /* tray stuff */
 gboolean tray_icon2_get_available(void);
@@ -153,4 +104,43 @@ void tray_icon2_create_tooltip(void);
 
 /* usefull for some stuff */
 extern GladeXML *pl3_xml;
+
+
+/**
+ * Playlist functions 
+ */
+
+/* Check if the playlist is fullscreen, this queries the _actual_ state */
+gboolean pl3_window_is_fullscreen(void);
+
+
+#ifdef DEBUG_TIMING
+/* Tic Tac system */
+#define TIMER_SUB(start,stop,diff)  diff.tv_usec = stop.tv_usec - start.tv_usec;\
+        diff.tv_sec = stop.tv_sec - start.tv_sec;\
+        if(diff.tv_usec < 0) {\
+            diff.tv_sec -= 1; \
+            diff.tv_usec += G_USEC_PER_SEC; \
+        }
+
+#define INIT_TIC_TAC() GTimeVal start123, stop123,diff123;\
+    g_get_current_time(&start123);
+
+#define TAC(a,ARGS...) g_get_current_time(&stop123);\
+    TIMER_SUB(start123, stop123, diff123);\
+    printf(a": %lu s, %lu us\n",##ARGS, (unsigned long)( diff123.tv_sec),(unsigned long)( diff123.tv_usec));    
+
+#define TOC(a,ARGS...) TAC(a,##ARGS);\
+    start123 = stop123;
+
+#else // DEBUG_TIMING
+
+
+#define INIT_TIC_TAC() ;
+#define TAC(a, ARGS...) ;
+#define TOC(a, ARGS...) ;
+
+#endif // DEBUG_TIMING
+#define TEC(a, ARGS...) TAC(a, ##ARGS)
+
 #endif
