@@ -10,6 +10,7 @@ typedef enum {
 	META_ARTIST_TXT 		= 8, 		/* Artist biography 	*/
 	META_SONG_TXT			= 16,		/* Lyrics 		*/
 	META_ARTIST_SIMILAR 	= 32,		/* Similar artists */
+    META_SONG_SIMILAR       = 64,       /* Similar songs */
 	META_QUERY_DATA_TYPES  	= 127, 		/* Bitmask for getting the metadata types only */
 	META_QUERY_NO_CACHE 	= 128		/* Do the query withouth checking the cache first */
 }MetaDataType;
@@ -22,6 +23,8 @@ typedef enum {
 
 typedef void (*MetaDataCallback)(mpd_Song *song, MetaDataResult result, char *path, gpointer data);
 
+void metadata_import_old_db(char *url);
+
 #include "gmpc-meta-watcher.h"
 extern GmpcMetaWatcher *gmw;
 
@@ -35,4 +38,9 @@ MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char
 
 void meta_data_cleanup(void);
 MetaDataResult meta_data_get_path(mpd_Song *tsong, MetaDataType type, gchar **path,MetaDataCallback callback, gpointer data);
+
+
+#define METADATA_DIR ".covers"
+gchar * gmpc_get_metadata_filename(MetaDataType  type, mpd_Song *song, char *extention);
+
 #endif
