@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
+ *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -119,195 +119,39 @@ void mpd_song_markup_escaped(char *buffer, int size, char *markup, mpd_Song *son
 /**
  * Code copied from gnome-screenshot
  */
-#define BLUR_RADIUS    5
-#define SHADOW_OFFSET  (BLUR_RADIUS * 4 / 5)
-#define SHADOW_OPACITY 0.5
-
-#define OUTLINE_RADIUS  1.1
-#define OUTLINE_OFFSET  0
-#define OUTLINE_OPACITY 1.0
-
-#define dist(x0, y0, x1, y1) sqrt(((x0) - (x1))*((x0) - (x1)) + ((y0) - (y1))*((y0) - (y1)))
-
-typedef struct {
-  int size;
-  double *data;
-} ConvFilter;
-
-static double
-gaussian (double x, double y, double r)
-{
-    return ((1 / (2 * M_PI * r)) *
-	    exp ((- (x * x + y * y)) / (2 * r * r)));
-}
-
-static ConvFilter *
-create_blur_filter (int radius)
-{
-  ConvFilter *filter;
-  int x, y;
-  double sum;
-  
-  filter = g_new0 (ConvFilter, 1);
-  filter->size = radius * 2 + 1;
-  filter->data = g_new (double, filter->size * filter->size);
-
-  sum = 0.0;
-  
-  for (y = 0 ; y < filter->size; y++)
-    {
-      for (x = 0 ; x < filter->size; x++)
-	{
-	  sum += filter->data[y * filter->size + x] = gaussian (x - (filter->size >> 1),
-								y - (filter->size >> 1),
-								radius);
-	}
-    }
-
-  for (y = 0; y < filter->size; y++)
-    {
-      for (x = 0; x < filter->size; x++)
-	{
-	  filter->data[y * filter->size + x] /= sum;
-	}
-    }
-
-  return filter;
-  
-}
-
-
-static GdkPixbuf *
-create_effect (GdkPixbuf *src, ConvFilter const *filter, int radius, int offset, double opacity)
-{
-  GdkPixbuf *dest;
-  int x, y, i, j;
-  int src_x, src_y;
-  int suma;
-  int dest_width, dest_height;
-  int src_width, src_height;
-  int src_rowstride, dest_rowstride;
-  gboolean src_has_alpha;
-  
-  guchar *src_pixels, *dest_pixels;
-
-  src_has_alpha =  gdk_pixbuf_get_has_alpha (src);
-
-  src_width = gdk_pixbuf_get_width (src);
-  src_height = gdk_pixbuf_get_height (src);
-  dest_width = src_width + 2 * radius + offset;
-  dest_height = src_height + 2 * radius + offset;
-
-  dest = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
-			 TRUE,
-			 gdk_pixbuf_get_bits_per_sample (src),
-			 dest_width, dest_height);
-  
-  gdk_pixbuf_fill (dest, 0);  
-  
-  src_pixels = gdk_pixbuf_get_pixels (src);
-  src_rowstride = gdk_pixbuf_get_rowstride (src);
-  
-  dest_pixels = gdk_pixbuf_get_pixels (dest);
-  dest_rowstride = gdk_pixbuf_get_rowstride (dest);
-  
-  for (y = 0; y < dest_height; y++)
-    {
-      for (x = 0; x < dest_width; x++)
-	{
-	  suma = 0;
-
-	  src_x = x - radius;
-	  src_y = y - radius;
-
-	  /* We don't need to compute effect here, since this pixel will be 
-	   * discarded when compositing */
-	  if (src_x >= 0 && src_x < src_width &&
-	      src_y >= 0 && src_y < src_height && 
-	      (!src_has_alpha ||
-	       src_pixels [src_y * src_rowstride + src_x * 4 + 3] == 0xFF))
-	    continue;
-
-	  for (i = 0; i < filter->size; i++)
-	    {
-	      for (j = 0; j < filter->size; j++)
-		{
-		  src_y = -(radius + offset) + y - (filter->size >> 1) + i;
-		  src_x = -(radius + offset) + x - (filter->size >> 1) + j;
-
-		  if (src_y < 0 || src_y >= src_height ||
-		      src_x < 0 || src_x >= src_width)
-		    continue;
-
-		  suma += ( src_has_alpha ? 
-			    src_pixels [src_y * src_rowstride + src_x * 4 + 3] :
-			    0xFF ) * filter->data [i * filter->size + j];
-		}
-	    }
-
-	  dest_pixels [y * dest_rowstride + x * 4 + 3] = CLAMP (suma * opacity, 0x00, 0xFF);
-	}
-    }
-  
-  return dest;
-}
-
-void
-screenshot_add_shadow (GdkPixbuf **src)
-{
-  GdkPixbuf *dest;
-  static ConvFilter *filter = NULL;
-  
-  if (!filter)
-    filter = create_blur_filter (BLUR_RADIUS);
-  
-  dest = create_effect (*src, filter, 
-			BLUR_RADIUS, SHADOW_OFFSET, SHADOW_OPACITY);
-
-  if (dest == NULL)
-	  return;
-
-  gdk_pixbuf_composite (*src, dest,
-			BLUR_RADIUS, BLUR_RADIUS,
-			gdk_pixbuf_get_width (*src),
-			gdk_pixbuf_get_height (*src),
-			BLUR_RADIUS, BLUR_RADIUS, 1.0, 1.0,
-			GDK_INTERP_NEAREST, 255);
-  g_object_unref (*src);
-  *src = dest;
-}
 
 void screenshot_add_border (GdkPixbuf **src)
 {
-	GdkPixbuf *pb = *src;
-        int x,y,width,height;
-        int pixel;
-        int n_channels = 0;
-        int rowstride = 0;
-        guchar *pixels;
-        if(!pb){
-                return;
-        }
-        
-        rowstride = gdk_pixbuf_get_rowstride(pb);	
-        n_channels = gdk_pixbuf_get_n_channels(pb);
-        width = gdk_pixbuf_get_width (pb);
-        height = gdk_pixbuf_get_height (pb);
-        pixels = gdk_pixbuf_get_pixels(pb);
+    GdkPixbuf *pb = *src;
+    int x,y,width,height;
+    int pixel;
+    int n_channels = 0;
+    int rowstride = 0;
+    guchar *pixels;
 
-        for(y=0;y<height;y++)
+    if(!pb){
+        return;
+    }
+
+    rowstride = gdk_pixbuf_get_rowstride(pb);	
+    n_channels = gdk_pixbuf_get_n_channels(pb);
+    width = gdk_pixbuf_get_width (pb);
+    height = gdk_pixbuf_get_height (pb);
+    pixels = gdk_pixbuf_get_pixels(pb);
+
+    for(y=0;y<height;y++)
+    {
+        for(x=0;x<width;x++)
         {
-                for(x=0;x<width;x++)
+            if(y == 0 || y == (height-1) || x == 0 || x == (width-1))
+            {
+                for(pixel=0; pixel < n_channels;pixel++)
                 {
-                        if(y == 0 || y == (height-1) || x == 0 || x == (width-1))
-                        {
-                                for(pixel=0; pixel < n_channels;pixel++)
-                                {
-                                        pixels[x*n_channels+y*rowstride+pixel] = 0;
-                                }
-                        }
+                    pixels[x*n_channels+y*rowstride+pixel] = 0;
                 }
+            }
         }
+    }
 }
 
 /**
@@ -399,47 +243,18 @@ char *gmpc_get_full_glade_path(char *filename)
 #endif
     return path;
 }
-static gint
-count_of_char_in_string (const gchar * string,
-			 const gchar c)
-{
-	int cnt = 0;
-	for(; *string; string++) {
-		if (*string == c) cnt++;
-	}
-	return cnt;
-}
-
-gchar *
-escape_single_quotes (const gchar * string)
-{
-	GString * gs;
-
-	if (string == NULL) {
-		return NULL;
-	}
-
-	if (count_of_char_in_string (string, '\'') == 0) {
-		return g_strdup(string);
-	}
-	gs = g_string_new ("");
-	for(; *string; string++) {
-		if (*string == '\'') {
-			g_string_append(gs, "'\\''");
-		}
-		else {
-			g_string_append_c(gs, *string);
-		}
-	}
-	return g_string_free (gs, FALSE);
-}
-
 
 void open_uri(const gchar *uri)
 {
 	int result;
 	gchar *command;
+#ifdef WIN32
+	gchar *browser_command = cfg_get_single_value_as_string_with_default(config, "Misc","browser-win32", "cmd /c start %s");
+#elif OSX
+	gchar *browser_command = cfg_get_single_value_as_string_with_default(config, "Misc","browser-osx", "open '%s'");
+#else
 	gchar *browser_command = cfg_get_single_value_as_string_with_default(config, "Misc","browser", "xdg-open '%s'");
+#endif
 	command	= g_strdup_printf(browser_command, uri);
 	result = g_spawn_command_line_async (command, NULL);
 	if(!result)
@@ -468,7 +283,8 @@ int *split_version(const char *uri)
 
 MpdData * misc_sort_mpddata(MpdData *data, GCompareDataFunc func, void *user_data)
 {
-    int i=0;
+	int j;
+	int i=0;
     MpdData_real *node;
     MpdData_real **nodes;
     
@@ -478,15 +294,15 @@ MpdData * misc_sort_mpddata(MpdData *data, GCompareDataFunc func, void *user_dat
 
     for(node = (MpdData_real*)mpd_data_get_first(data);node;node =(MpdData_real *)mpd_data_get_next_real((MpdData *)node, FALSE))i++;
 
-    nodes = g_malloc0(i*sizeof(*node));
-    int j;
+    nodes = g_malloc0(i*sizeof(MpdData_real *));
+    
     node = (MpdData_real *)mpd_data_get_first(data);
     for(j=0;j<i;j++)
     {
         nodes[j] = node;
         node =(MpdData_real *) mpd_data_get_next_real((MpdData *)node, FALSE);
     }
-    g_qsort_with_data(nodes, i, sizeof(MpdData_real *), func,user_data);
+    g_qsort_with_data(nodes, i, sizeof(*nodes), func,user_data);
     nodes[0]->prev = NULL;
     nodes[0]->next= NULL;
     nodes[0]->first=nodes[0];
@@ -553,6 +369,17 @@ static gint __add_sort(gpointer aa, gpointer bb, gpointer data)
     }
     if(a->type == MPD_DATA_TYPE_SONG && b->type == MPD_DATA_TYPE_SONG)
     {
+        if(a->song->date == NULL && b->song->date != NULL)
+            return -1;
+        else if(b->song->date == NULL && a->song->date != NULL)
+            return 1;
+        else if (a->song->date  && b->song->date)
+        {
+            int compv = atoi(a->song->date)-atoi(b->song->date); 
+            if(compv != 0)
+                return compv;
+        }
+        /*
         if(a->song->artist == NULL && b->song->artist != NULL)
             return -1;
         else if(b->song->artist == NULL && a->song->artist != NULL)
@@ -565,6 +392,7 @@ static gint __add_sort(gpointer aa, gpointer bb, gpointer data)
                 return compv;
             }
         }
+        */
         if(a->song->album == NULL && b->song->album != NULL)
             return -1;
         else if(b->song->album == NULL && a->song->album != NULL)
@@ -654,3 +482,52 @@ MpdData *misc_mpddata_remove_duplicate_songs(MpdData *data)
     return (MpdData *)node->first;
 }
 
+
+gchar ** tokenize_string(const gchar *string)
+{
+    gchar ** result = NULL; 	/* the result with tokens 		*/
+	int i = 0;		/* position in string 			*/
+	int br = 0;		/* number for open ()[]'s		*/
+	int bpos = 0;		/* begin position of the cur. token 	*/
+    int bstop = 0;
+	int tokens=0;
+    int length = 0;
+	if(string == NULL) return NULL;
+    length = strlen(string);
+	for(i=0; i <= length;i++)
+	{
+		/* check for opening  [( */
+		if((br !=0 || bpos == i) && (string[i] == '(' || string[i] == '[' || string[i] == '{')){
+            if(!br && bpos == i)
+                bpos = i+1;
+            br++;
+        }
+		/* check closing */
+		else if(br && (string[i] == ')' || string[i] == ']' || string[i] == '}')){
+            br--;
+            bstop = i;
+            if(!br && (string[i+1] == ' ' || string[i+1] == '\0')) { 
+                i++;
+            }
+        }
+        else 
+            bstop=i;
+        /* if multiple spaces at begin of token skip them */
+		if(string[i] == ' ' && !(i-bpos))bpos++;
+		/* if token end or string end add token to list */
+		else if((string[i] == ' ' && !br) || string[i] == '\0')
+		{
+            if((bstop-bpos) >0)
+            {
+                result = g_realloc(result,(tokens+2)*sizeof(gchar *));
+                result[tokens] = g_strndup(&string[bpos], bstop-bpos);
+                result[tokens+1] = NULL;
+                bpos = i+1;
+                bstop = bpos;
+                tokens++;
+            }
+		}
+
+	}
+	return result;
+}
