@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
  * Borrowed from Lee Willis <lee@leewillis.co.uk> that
  * Borrowed heavily from code by Jan Arne Petersen <jpetersen@uni-bonn.de>
  *
@@ -64,6 +64,7 @@ enum {
 	MM_VOLUME_UP,
 	MM_VOLUME_DOWN,
 	MM_SHOW_NOTIFICATION,
+    MM_TOGGLE_MUTE,
 	LAST_SIGNAL
 };
 
@@ -82,6 +83,7 @@ char *keynames[LAST_SIGNAL] = {
 	N_("Volume Up"),	/** MM_VOLUME_UP */
 	N_("Volume Down"),	/** MM_VOLUME_DOWN */
 	N_("Show song"),	/** MM_SHOW_NOTIFICATION */
+    N_("Toggle Mute"),  /** MM_TOGGLE_MUTE */
 };	
 
 static GObjectClass *parent_class;
@@ -235,6 +237,17 @@ static void mmkeys_class_init (MmKeysClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__INT,	
 				G_TYPE_NONE, 1, G_TYPE_INT); 	
+	signals[MM_TOGGLE_MUTE] = 
+		g_signal_new ("mm_toggle_mute",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				0,
+				NULL, NULL,
+				g_cclosure_marshal_VOID__INT,	
+				G_TYPE_NONE, 1, G_TYPE_INT); 	
+
+
+
 }
 
 static void mmkeys_finalize (GObject *object)
@@ -423,26 +436,8 @@ static GdkFilterReturn filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpoint
 			return GDK_FILTER_REMOVE;
 		}
 	}
-	/*
-	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {
-		g_signal_emit (data, signals[MM_PLAYPAUSE], 0, 0);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {
-		g_signal_emit (data, signals[MM_PLAYPAUSE], 0, 0);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
-		g_signal_emit (data, signals[MM_PREV], 0, 0);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
-		g_signal_emit (data, signals[MM_NEXT], 0, 0);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop) == key->keycode) {
-		g_signal_emit (data, signals[MM_STOP], 0, 0);
-		return GDK_FILTER_REMOVE;
-	} else {
-	*/
-		return GDK_FILTER_CONTINUE;
-	/*}*/
+
+    return GDK_FILTER_CONTINUE;
 }
 
 
@@ -534,26 +529,16 @@ void mmkeys_pref_construct(GtkWidget *container);
 GladeXML *mmkeys_pref_xml = NULL;
 
 gmpcPrefPlugin mmkeys_gpp = {
-	mmkeys_pref_construct,
-	mmkeys_pref_destroy
+	.construct      = mmkeys_pref_construct,
+	.destroy        = mmkeys_pref_destroy
 };
 
 gmpcPlugin mmkeys_plug = 
  {
-	N_("Multimedia Keys"),
-	{1,1,1},
-	GMPC_INTERNALL,
-	0,
-	NULL,           /* path */
-	NULL,           /* init */
-	NULL,           /* Destroy */
-	NULL,           /* browser ext */
-	NULL,           /* status changed */
-        NULL,           /* connection changed */
-	&mmkeys_gpp,    /* preferences */
-	NULL,           /* metadata */
-	NULL,           /* get_enable*/
-	NULL            /* set enable */
+	. name          = N_("Multimedia Keys"),
+	.version        = {1,1,1},
+	.plugin_type    = GMPC_INTERNALL,
+	.pref           = &mmkeys_gpp    /* preferences */
 };
 
 static void accel_cleared_callback(GtkCellRendererText *cell, const char *path_string, gpointer data)
@@ -608,12 +593,13 @@ accel_edited_callback (GtkCellRendererText *cell,
 			keycodes[i] == hardware_keycode &&
 			masks[i] == mask)
 		{
+			gchar *message;
 			gchar *rawkeysym = egg_virtual_accelerator_name (
 				XKeycodeToKeysym(GDK_DISPLAY(), hardware_keycode, 0),
 				hardware_keycode, mask);
 			gchar *keysym = g_markup_escape_text(rawkeysym, -1);
 			q_free (rawkeysym);
-			gchar *message = g_strdup_printf( _("<b>Duplicate mapping detected</b>\n\n"
+			message = g_strdup_printf( _("<b>Duplicate mapping detected</b>\n\n"
 					"%s is already mapped to %s"),
 					keysym, _(keynames[i]) );
 			q_free (keysym);
@@ -636,12 +622,13 @@ accel_edited_callback (GtkCellRendererText *cell,
 	gtk_tree_path_free (path);
 	if( keyerror[key] )
 	{
+		gchar *message;
 		gchar *rawkeysym = egg_virtual_accelerator_name (
 			XKeycodeToKeysym(GDK_DISPLAY(), keycodes[key], 0),
 			keycodes[key], masks[key]);
 		gchar *keysym = g_markup_escape_text(rawkeysym, -1);
 		q_free (rawkeysym);
-		gchar *message = g_strdup_printf(
+		message = g_strdup_printf(
 			_("Could not grab multimedia key:\n\n"
 			"\t%s: %s\n\n"
 			"Ensure that your window manager (or other applications) have not already bound this key for some other function, then restart gmpc."),
