@@ -6,11 +6,11 @@
 #include "config1.h"
 #include "mpdinteraction.h"
 
-static int ignore = FALSE;
-G_LOCK_DEFINE (connecting_lock);
+//G_LOCK_DEFINE (connecting_lock);
 
-/* old stuff */
-static void preferences_update(void);
+static int current_volume = 0;
+
+static GMutex *connecting_lock = NULL;//g_mutex_new();//G_STATIC_MUTEX_INIT;
 
 /* Server Settings plugin */
 static void server_pref_construct(GtkWidget *);
@@ -20,7 +20,6 @@ static void server_pref_destroy(GtkWidget *);
 static void connection_pref_construct(GtkWidget *container);
 static void connection_pref_destroy(GtkWidget *container);
 
-static void ServerConnectionChangedCallback(MpdObj *mi, int connected, gpointer data);
 static void ServerStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
 
 static GladeXML *server_pref_xml = NULL;
@@ -38,13 +37,13 @@ void auth_enable_toggled(GtkToggleButton *but);
 void preferences_window_autoconnect(GtkToggleButton *tog);
 void preferences_window_connect(GtkWidget *but);
 void preferences_window_disconnect(GtkWidget *but);
-void update_preferences_name(void);
-void update_preferences_hostname(void);
-void update_preferences_portnumber(void);
-void update_preferences_information(void);
+void update_preferences_name(GtkWidget *entry);
+void update_preferences_hostname(GtkWidget *entry);
+void update_preferences_portnumber(GtkWidget *wid);
+void update_preferences_information(GtkWidget *wid);
 void connection_profiles_changed(GtkComboBox *combo, gpointer data);
-void connection_add_profile(void);
-void connection_remove_profile(void);
+void connection_add_profile(GtkWidget *but);
+void connection_remove_profile(GtkWidget *but);
 void submenu_artist_clicked(GtkWidget *item);
 void submenu_album_clicked(GtkWidget *item);
 void submenu_genre_clicked(GtkWidget *item);
@@ -59,8 +58,8 @@ gmpcPlugin server_plug = {
 	NULL,				/** init */
         NULL,                           /** Destroy */
 	NULL,				/** browser ext */
-	&ServerStatusChangedCallback,	/** status changed */
-	&ServerConnectionChangedCallback,	/** connection changed */
+	ServerStatusChangedCallback,	/** status changed */
+	NULL,
 	&server_gpp,			/** preferences */
 	NULL,				/** Metadata */
 	NULL,				/** get enabled */
@@ -74,7 +73,7 @@ enum
 	N_COLUMNS
 };
 
-GladeXML *connection_pref_xml = NULL;
+
 gmpcPrefPlugin connection_gpp = {
 	connection_pref_construct,
 	connection_pref_destroy
@@ -124,11 +123,12 @@ static int connected_to_mpd(mpd_Connection *mpd_conn)
 	gtk_widget_hide(glade_xml_get_widget(pl3_xml, "pl3_progressbar"));
 	g_source_remove(connecting_pulse);
 	connecting_pulse = 0;
+	g_mutex_unlock(connecting_lock);
 	if(connection)
 	{
 		mpd_connect_real(connection, mpd_conn);
 	}
-    G_UNLOCK(connecting_lock);
+
 	return FALSE;
 }
 static void connection_thread(void)
@@ -142,7 +142,10 @@ static void connection_thread(void)
 int connect_to_mpd()
 {
 	char *string = NULL;
-    if(!G_TRYLOCK(connecting_lock))
+    if(connecting_lock == NULL){
+        connecting_lock = g_mutex_new();
+    }
+    if(!/*G_TRYLOCK*/g_mutex_trylock(connecting_lock))
     {
         debug_printf(DEBUG_ERROR, "Allready busy connecting to mpd.. not doing anything");
         return FALSE;
@@ -192,17 +195,6 @@ int connect_to_mpd()
 	return FALSE;
 }
 
-/* DEFAULT FUNCTIONS */
-static void ServerConnectionChangedCallback(MpdObj *mi, int connected, gpointer data)
-{
-	if(connected)
-	{
-		connect_callback(mi);
-	}
-	preferences_update();
-}
-
-
 /******************************************************
  * PLAYER FUNCTIONS
  */
@@ -297,6 +289,29 @@ void volume_down()
 {
 	mpd_status_set_volume(connection, mpd_status_get_volume(connection)-5);
 }
+
+
+void volume_mute()
+{
+    mpd_status_set_volume(connection, 0);
+}
+
+void volume_unmute()
+{
+    if(mpd_status_get_volume(connection) == 0)
+        mpd_status_set_volume(connection, current_volume);
+}
+
+void volume_toggle_mute()
+{
+    if(current_volume > 0 && mpd_status_get_volume(connection) == 0)
+    {
+        mpd_status_set_volume(connection, current_volume);
+    }else {
+        mpd_status_set_volume(connection, 0);
+    }
+}
+
 /*****************************************************************
  * Preferences
  */
@@ -319,7 +334,7 @@ static void outputs_toggled(GtkCellRendererToggle *cell, gchar *path_str, GtkTre
 	gtk_tree_path_free(path);
 }
 
-static void create_outputs_tree()
+static void create_outputs_tree(void)
 {
 	GtkListStore *model;
 	GtkCellRenderer *cell;
@@ -350,7 +365,7 @@ static void create_outputs_tree()
 	gtk_tree_view_column_set_visible(col, FALSE);
 	gtk_tree_view_append_column(tree, col);
 }
-static void update_outputs_settings()
+static void update_outputs_settings(void)
 {
 	GtkTreeIter iter;
 	GtkListStore *store;
@@ -408,7 +423,7 @@ void xfade_time_changed(GtkSpinButton *but)
 	}
 	mpd_status_set_crossfade(connection, fade_time);
 }
-static void xfade_update()
+static void xfade_update(void)
 {
 	if(mpd_status_get_crossfade(connection) > 0)
 	{
@@ -530,11 +545,22 @@ void add_directory(const gchar *path)
 
 void ServerStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata)
 {
+    if(what&MPD_CST_VOLUME)
+    {
+        int volume = mpd_status_get_volume(connection);
+        if(volume)
+            current_volume =  volume;
+    }
+
 	if(!server_pref_xml)return;
 	if(what&MPD_CST_CROSSFADE)
 	{
 		xfade_update();
 	}
+    if(what&MPD_CST_OUTPUT)
+    {
+        update_outputs_settings();
+    }
 }
 
 
@@ -596,36 +622,73 @@ static void server_pref_construct(GtkWidget *container)
 /**************************************************
  * Connection Preferences *
  */
-/* this function is called from the main loop, it makes sure stuff is up-to-date(r) */
-
-
-
-
-static void preferences_update()
+static void gmpc_profiles_changed_pref_win(GmpcProfiles *prof,const int changed, const int col, const char * id, GladeXML *xml)
 {
-	if(connection_pref_xml == NULL) return;
-	if(!mpd_check_connected(connection))
+	if(changed == PROFILE_ADDED)
 	{
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), TRUE);
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), FALSE);
+		GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
+		GtkTreeIter iter;
+		const char *name = gmpc_profiles_get_name(prof, id);
+		GtkTreeModel *store = gtk_combo_box_get_model(combo);
+		gtk_list_store_append(GTK_LIST_STORE(store), &iter);
+		gtk_list_store_set(GTK_LIST_STORE(store), &iter, 0, id, 1, name,-1);
+	}
+	if(changed == PROFILE_REMOVED)
+	{
+		/* TODO: */
+
+	}
+	if(changed == PROFILE_COL_CHANGED)
+	{
+		GtkTreeIter iter;
+		GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
+		GtkListStore *store = (GtkListStore *)gtk_combo_box_get_model(combo);
+		/* tell it to update all the information in the view. 
+		 * might be to much work, so check id*/
+		gchar *uid;
+		if(gtk_combo_box_get_active_iter(combo,&iter) && id)
+		{
+
+			gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, -1);
+			if(strcmp(uid, id) == 0)
+			{
+				connection_profiles_changed(combo, NULL);
+			}
+			g_free(uid);
+		}
+	}
+}
+
+static void gmpc_connection_changed_pref_win(GmpcConnection *object, MpdObj *mi, int connected, GladeXML *xml)
+{ 
+    debug_printf(DEBUG_INFO, "set buttons %i", connected);
+    if(connected != mpd_check_connected(mi)) return;
+	if(!connected)
+	{
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "bt_con"), TRUE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "bt_dis"), FALSE);
 	}
 	else
 	{
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), FALSE);
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), TRUE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "bt_con"), FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "bt_dis"), TRUE);
 	}
 }
 void entry_auth_changed(GtkEntry *entry)
 {
-	GtkComboBox *combo = (GtkComboBox *)glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(entry));
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
+	GtkComboBox *combo = (GtkComboBox *)glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkListStore *store = (GtkListStore *)gtk_combo_box_get_model(combo);
-	if(ignore) return;
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		gchar *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
-    gmpc_profiles_set_password(gmpc_profiles, uid, (char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "entry_auth"))));
+		g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
+		gmpc_profiles_set_password(gmpc_profiles, uid, (char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry_auth"))));
+		g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
 		q_free(uid);
 		q_free(value);
 	}
@@ -633,85 +696,107 @@ void entry_auth_changed(GtkEntry *entry)
 
 void auth_enable_toggled(GtkToggleButton *but)
 {
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(but));
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
-	if(ignore) return;
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
-    gmpc_profiles_set_do_auth(gmpc_profiles, uid, gtk_toggle_button_get_active(but));
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "entry_auth"), 
+		g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
+
+		gmpc_profiles_set_do_auth(gmpc_profiles, uid, gtk_toggle_button_get_active(but));
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "entry_auth"), 
 				gmpc_profiles_get_do_auth(gmpc_profiles, uid));	
+
+		g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
 		q_free(uid);
 		q_free(value);
 	}
 }
 
-void update_preferences_name(void)
+void update_preferences_name(GtkWidget *entry)
 {
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(entry));
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
-	if(ignore) return;
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
+		g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
+
 		gmpc_profiles_set_name(gmpc_profiles, uid,
-				(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "name_entry"))));
+				(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "name_entry"))));
+		g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
+
 		q_free(uid);
 		q_free(value);
-		value = (char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "name_entry")));
+		value = (char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "name_entry")));
 		gtk_list_store_set(GTK_LIST_STORE(store), &iter, 1, value, -1);
 	}
 }
 
-void update_preferences_hostname(void)
+void update_preferences_hostname(GtkWidget *entry)
 {
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(entry));
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
-	if(ignore) return;
 
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
+		g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
+
 		gmpc_profiles_set_hostname(gmpc_profiles, uid,
-				(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "hostname_entry"))));
+				(char *)gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "hostname_entry"))));
+		g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
+
 		q_free(uid);
 		q_free(value);
 	}
 }
-void update_preferences_portnumber(void)
+void update_preferences_portnumber(GtkWidget *wid)
 {
-
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(wid));
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
-	if(ignore) return;
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
+		g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
 		gmpc_profiles_set_port(gmpc_profiles, uid,
-				gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget(connection_pref_xml, "port_spin"))));
+				gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "port_spin"))));
+		g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
 		q_free(uid);
 		q_free(value);
 	}
 }
-void update_preferences_information(void)
+void update_preferences_information(GtkWidget *wid)
 {
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(wid));
 	cfg_set_single_value_as_float(config,"connection", "timeout",
-			(float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(connection_pref_xml, "timeout_spin"))));
+			(float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"timeout_spin"))));
 
 	/* update timeout live */
 	if(mpd_check_connected(connection))
 	{
 		mpd_set_connection_timeout(connection, 
-				(float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(connection_pref_xml, "timeout_spin"))));
+				(float)gtk_spin_button_get_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "timeout_spin"))));
 	}
 }
 
@@ -722,9 +807,10 @@ void preferences_window_autoconnect(GtkToggleButton *tog)
 
 void preferences_window_connect(GtkWidget *but)
 {
-	if(connection_pref_xml)
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(but));
+	if(xml)
 	{
-		GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+		GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 		GtkTreeIter iter;
 		GtkTreeModel *store = gtk_combo_box_get_model(combo);
 		if(gtk_combo_box_get_active_iter(combo,&iter))
@@ -743,10 +829,10 @@ void preferences_window_connect(GtkWidget *but)
 		}
 	}
 	else {
-			if(!mpd_check_connected(connection))
-			{
-				if(!connect_to_mpd());
-			}  
+		if(!mpd_check_connected(connection))
+		{
+			if(!connect_to_mpd());
+		}  
 	}
 }
 
@@ -760,17 +846,24 @@ void preferences_window_disconnect(GtkWidget *but)
 
 static void connection_pref_destroy(GtkWidget *container)
 {
-	if(connection_pref_xml)
+	GtkWidget *widget = gtk_bin_get_child(GTK_BIN(container));
+	if(widget)
 	{
-		GtkWidget *vbox = glade_xml_get_widget(connection_pref_xml, "connection-vbox");
-		gtk_container_remove(GTK_CONTAINER(container),vbox);
-		g_object_unref(connection_pref_xml);
-		connection_pref_xml = NULL;
+		GladeXML *xml = glade_get_widget_tree(widget);
+		if(xml)
+		{
+			gtk_container_remove(GTK_CONTAINER(container),widget);
+			g_object_unref(xml);
+		}
+	}
+	else
+	{
+		debug_printf(DEBUG_ERROR, "No child widget to destroy\n");
 	}
 }
-
 void connection_profiles_changed(GtkComboBox *combo, gpointer data)
 {
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(combo));
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
 	if(gtk_combo_box_get_active_iter(combo,&iter))
@@ -778,96 +871,119 @@ void connection_profiles_changed(GtkComboBox *combo, gpointer data)
 		char *value= NULL, *uid = NULL, *string;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
 
-		ignore = TRUE;
 
 		/**
 		 * Set name
 		 */
-		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "name_entry")), value);
+		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "name_entry")), value);
 		/**
 		 * Set hostname
 		 */
 		string = g_strdup(gmpc_profiles_get_hostname(gmpc_profiles, uid));
-		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "hostname_entry")), 
+		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "hostname_entry")), 
 				string);
 		g_free(string);
 		/**
 		 * Set port number 
 		 */
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(connection_pref_xml, "port_spin")), 
-          gmpc_profiles_get_port(gmpc_profiles, uid));
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "port_spin")), 
+				gmpc_profiles_get_port(gmpc_profiles, uid));
 
 		/**
 		 * Set password check, and entry
 		 */
 		string = g_strdup(gmpc_profiles_get_password(gmpc_profiles, uid));
 		gtk_toggle_button_set_active((GtkToggleButton *)
-				glade_xml_get_widget(connection_pref_xml, "ck_auth"), 
-          gmpc_profiles_get_do_auth(gmpc_profiles, uid));
-		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "entry_auth"), 
-          gmpc_profiles_get_do_auth(gmpc_profiles, uid));
+				glade_xml_get_widget(xml, "ck_auth"), 
+				gmpc_profiles_get_do_auth(gmpc_profiles, uid));
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "entry_auth"), 
+				gmpc_profiles_get_do_auth(gmpc_profiles, uid));
 
-		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(connection_pref_xml, "entry_auth")),/*string);*/
-          string);
-    g_free(string);
+		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry_auth")),/*string);*/
+			string);
+		g_free(string);
 
 		/**
 		 * Only enable the rmeove button when there is more then 1 profile
 		 */
 		if(gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store),NULL) >1)
 		{
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "remove_butt"), TRUE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml, "remove_butt"), TRUE);
 		}
 		else
 		{
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "remove_butt"), FALSE);
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml, "remove_butt"), FALSE);
 		}
 		q_free(value);
 		q_free(uid);
-		ignore = FALSE;
 	}
 
 }
 
-void connection_add_profile(void)
+void connection_add_profile(GtkWidget *but)
 {
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	gchar *value = NULL;
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(but));
+	GtkWidget *vbox = glade_xml_get_widget(xml, "connection-vbox");
+	gulong *a = g_object_get_data(G_OBJECT(vbox),"profile-signal-handler"); 
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
-	gchar *value = gmpc_profiles_create_new_item(gmpc_profiles, NULL); /*g_strdup_printf("%u", g_random_int());*/
+
+	g_signal_handler_block(G_OBJECT(gmpc_profiles), *a);
+	value = gmpc_profiles_create_new_item(gmpc_profiles, NULL); /*g_strdup_printf("%u", g_random_int());*/
 	gtk_list_store_append(GTK_LIST_STORE(store), &iter);
 	gtk_list_store_set(GTK_LIST_STORE(store), &iter, 0, value, 1, "Name", -1);
-	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(glade_xml_get_widget(connection_pref_xml, "cb_profiles")),&iter);
+	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(glade_xml_get_widget(xml, "cb_profiles")),&iter);
+	g_signal_handler_unblock(G_OBJECT(gmpc_profiles), *a);
 }
 
-void connection_remove_profile(void)
+void connection_remove_profile(GtkWidget *but)
 {
-	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(connection_pref_xml, "cb_profiles");
+	GladeXML *xml = glade_get_widget_tree(GTK_WIDGET(but));
+	GtkComboBox *combo = (GtkComboBox *) glade_xml_get_widget(xml, "cb_profiles");
 	GtkTreeIter iter;
 	GtkTreeModel *store = gtk_combo_box_get_model(combo);
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
-    gmpc_profiles_remove_item(gmpc_profiles, uid);
+		gmpc_profiles_remove_item(gmpc_profiles, uid);
 		gtk_list_store_remove(GTK_LIST_STORE(store), &iter);
 		q_free(uid);
 		q_free(value);
-		gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(connection_pref_xml, "cb_profiles")),0);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "cb_profiles")),0);
 	}
 }
+static void destroy_profile_signal_handler(gpointer box)
+{
+	gulong *a = box;
+	g_signal_handler_disconnect(G_OBJECT(gmpc_profiles), *a);
+	g_free(a);
+}
+static void destroy_connection_signal_handler(gpointer box)
+{
+	gulong *a = box;
+	g_signal_handler_disconnect(G_OBJECT(gmpcconn), *a);
+	g_free(a);
+}
+
+
+
 
 static void connection_pref_construct(GtkWidget *container)
 {
+	gulong *a;
 	gchar *def_profile = NULL;
 	GList *mult, *iter;
+	GtkWidget *vbox = NULL;
 	GtkCellRenderer *renderer = NULL;
 	GtkListStore *store = NULL;
 	gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-	connection_pref_xml = glade_xml_new(path, "connection-vbox",NULL);
+	GladeXML *connection_pref_xml = glade_xml_new(path, "connection-vbox",NULL);
 	q_free(path);
 
-
+	vbox = glade_xml_get_widget(connection_pref_xml, "connection-vbox");
 	/**
 	 * Profile selector
 	 * uid, name
@@ -886,7 +1002,7 @@ static void connection_pref_construct(GtkWidget *container)
 		iter = mult;
 		do{
 			GtkTreeIter piter;
-      gchar *value = gmpc_profiles_get_name(gmpc_profiles, (char *)iter->data); 
+			const gchar *value = gmpc_profiles_get_name(gmpc_profiles, (char *)iter->data); 
 			gtk_list_store_append(store, &piter);
 			gtk_list_store_set(store, &piter, 0,iter->data, 1,value,-1);
 			if(!strcmp((char *)(iter->data), def_profile))
@@ -895,19 +1011,19 @@ static void connection_pref_construct(GtkWidget *container)
 			}
 			i++;
 		}while((iter = g_list_next(iter)));
-    g_list_foreach(mult, (GFunc)g_free, NULL);
-    g_list_free(mult);
+		g_list_foreach(mult, (GFunc)g_free, NULL);
+		g_list_free(mult);
 	}
 	else{
-	/*	GtkTreeIter piter;
-		gchar *value = cfg_get_single_value_as_string_with_default(profiles, "Default", "name", "Default");
-		gtk_list_store_append(store, &piter);
-		gtk_list_store_set(store, &piter, 0,"Default", 1,value,-1);
-		q_free(value);
-		gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(connection_pref_xml, "cb_profiles")),0);
-*/
+		/*	GtkTreeIter piter;
+			gchar *value = cfg_get_single_value_as_string_with_default(profiles, "Default", "name", "Default");
+			gtk_list_store_append(store, &piter);
+			gtk_list_store_set(store, &piter, 0,"Default", 1,value,-1);
+			q_free(value);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(connection_pref_xml, "cb_profiles")),0);
+			*/
 	}
-  q_free(def_profile);
+	q_free(def_profile);
 
 	connection_profiles_changed(GTK_COMBO_BOX(glade_xml_get_widget(connection_pref_xml, "cb_profiles")),NULL);
 
@@ -917,23 +1033,32 @@ static void connection_pref_construct(GtkWidget *container)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(connection_pref_xml, "ck_autocon")), 
 			cfg_get_single_value_as_int_with_default(config,"connection", "autoconnect", 0));
 
-	if(connection_pref_xml)
+
+	/* set the right sensitive stuff */
+	if(!mpd_check_connected(connection))
 	{
-		GtkWidget *vbox = glade_xml_get_widget(connection_pref_xml, "connection-vbox");
-		/* set the right sensitive stuff */
-		if(!mpd_check_connected(connection))
-		{
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), TRUE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), FALSE);	    
-		}
-		else
-		{
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), TRUE);	    
-		}
-		gtk_container_add(GTK_CONTAINER(container),vbox);
-		glade_xml_signal_autoconnect(connection_pref_xml);
+		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), TRUE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), FALSE);	    
 	}
+	else
+	{
+		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_con"), FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(connection_pref_xml, "bt_dis"), TRUE);	    
+	}
+	gtk_container_add(GTK_CONTAINER(container),vbox);
+	glade_xml_signal_autoconnect(connection_pref_xml);
+
+
+	a = g_malloc0(sizeof(*a));
+	*a= g_signal_connect(G_OBJECT(gmpc_profiles), "changed",
+			G_CALLBACK(gmpc_profiles_changed_pref_win), connection_pref_xml);
+	g_object_set_data_full(G_OBJECT(vbox), "profile-signal-handler", a, destroy_profile_signal_handler);
+
+	a = g_malloc0(sizeof(*a));
+	*a= g_signal_connect(G_OBJECT(gmpcconn), "connection-changed",
+			G_CALLBACK(gmpc_connection_changed_pref_win), connection_pref_xml);
+	g_object_set_data_full(G_OBJECT(vbox), "conn-signal-handler", a, destroy_connection_signal_handler);
+
 }
 void connection_set_current_profile(const char *uid)
 {
@@ -978,18 +1103,18 @@ char *connection_get_hostname()
 }
 int connection_get_port()
 {
-  int retv;
+	int retv;
 	gchar *profile = gmpc_profiles_get_current(gmpc_profiles);
 	retv  = gmpc_profiles_get_port(gmpc_profiles, profile);
-  q_free(profile);
+	q_free(profile);
 	return retv;
 }
 char *connection_get_password()
 {
 	gchar *profile = gmpc_profiles_get_current(gmpc_profiles);
 	gchar *retv  = gmpc_profiles_get_password(gmpc_profiles, profile);
-  g_free(profile);
-  return retv;
+	g_free(profile);
+	return retv;
 }
 
 /**
