@@ -1,3 +1,4 @@
+#include <string.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gmodule.h>
@@ -10,7 +11,9 @@ int num_plugins = 0;
 
 gmpcPlugin * plugin_get_from_id(int id)
 {
-	return plugins[plugin_get_pos(id)];
+    int pos = plugin_get_pos(id);
+    g_assert_cmpint(pos, <, num_plugins);
+	return plugins[pos];
 }
 
 
@@ -20,6 +23,7 @@ int plugin_get_pos(int id)
 }
 static int plugin_validate(gmpcPlugin *plug)
 {
+    int i;
     if(plug == NULL)
     {
         debug_printf(DEBUG_ERROR, "plug != NULL failed");
@@ -30,6 +34,14 @@ static int plugin_validate(gmpcPlugin *plug)
         debug_printf(DEBUG_ERROR, "Plugin has no name.");
         return FALSE;
     }
+    for(i=0;i<num_plugins;i++)
+    {
+        if(strcmp(gmpc_plugin_get_name(plugins[i]), plug->name) == 0)
+        {
+            debug_printf(DEBUG_ERROR, "Plugin with same name allready exists.");
+            return FALSE;
+        }
+    }
     if(plug->set_enabled == NULL || plug->get_enabled == NULL)
     {
         debug_printf(DEBUG_ERROR, "%s: set_enabled == NULL || get_enabled == NULL failed",plug->name);
@@ -55,6 +67,11 @@ static int plugin_validate(gmpcPlugin *plug)
             debug_printf(DEBUG_ERROR, "%s: plugin_type&GMPC_PLUGIN_META_DATA && plugin->metadata->get_priority != NULL Failed",plug->name);
             return FALSE;                                                                                             
         }
+        if(plug->metadata->set_priority == NULL)
+        {
+            debug_printf(DEBUG_ERROR, "%s: plugin_type&GMPC_PLUGIN_META_DATA && plugin->metadata->set_priority != NULL Failed",plug->name);
+            return FALSE;                                                                                             
+        }
         if(plug->metadata->get_image == NULL)
         {   
             debug_printf(DEBUG_ERROR, "%s: plugin_type&GMPC_PLUGIN_META_DATA && plugin->metadata->get_image != NULL Failed",plug->name);
@@ -135,8 +152,9 @@ static int plugin_load(char *path, const char *file)
 				file, *api_version, PLUGIN_API_VERSION);
 		debug_printf(DEBUG_ERROR, "Plugin '%s' has the wrong api version.\nPlugin api is %i, but we need %i",
 			       file, *api_version, PLUGIN_API_VERSION);
-		show_error_message(message,FALSE);
-		q_free(string);
+	//	show_error_message(message,FALSE);
+        playlist3_show_error_message(message, ERROR_WARNING);
+        q_free(string);
 		q_free(message);
 		g_module_close(handle);
 		return 1;
@@ -145,8 +163,9 @@ static int plugin_load(char *path, const char *file)
 	if(!g_module_symbol(handle, "plugin", (gpointer)&plug)){
 		gchar *message = g_strdup_printf("Failed to load plugin:\n%s:%s",file, g_module_error());
 		debug_printf(DEBUG_ERROR, "plugin_load: symbol failed to bind: %s:%s\n",file, g_module_error());
-		show_error_message(message,FALSE);
-		q_free(string);
+//		show_error_message(message,FALSE);
+        playlist3_show_error_message(message, ERROR_CRITICAL);
+        q_free(string);
 		q_free(message);
 		g_module_close(handle);
 		return 1;
@@ -169,10 +188,16 @@ static int plugin_load(char *path, const char *file)
     plugin_add(plug,1);
     return 0;
 }
+static gboolean __show_plugin_load_error(gpointer data)
+{
+    playlist3_show_error_message(_("One or more plugins failed to load, see help->messages for more information"), ERROR_CRITICAL);
+    return FALSE;
+}
 
 void plugin_load_dir(gchar *path)
 {
     GDir *dir = g_dir_open(path, 0, NULL);
+    int failure = 0;
     if(dir)
     {
         const gchar *dirname = NULL;
@@ -184,6 +209,7 @@ void plugin_load_dir(gchar *path)
             {
                 if(plugin_load(path,dirname))
                 {
+                    failure = 1;
                     debug_printf(DEBUG_ERROR, "Failed to load plugin: %s\n", dirname);
                 }
             }
@@ -195,4 +221,86 @@ void plugin_load_dir(gchar *path)
         }
         g_dir_close(dir);
     }
+    if(failure)
+    {
+        gtk_init_add(__show_plugin_load_error, NULL);
+    }
+}
+
+/**
+ * gmpcPlugin
+ */
+
+void gmpc_plugin_destroy(gmpcPlugin *plug)
+{
+    if(plug->destroy)
+    {
+        plug->destroy();
+    }
+}
+void gmpc_plugin_init(gmpcPlugin *plug)
+{
+    if(plug->init)
+    {
+        plug->init();
+    }
+}
+void gmpc_plugin_status_changed(gmpcPlugin *plug, MpdObj *mi, ChangedStatusType what)
+{
+    if(plug->mpd_status_changed)
+    {
+        plug->mpd_status_changed(mi,what,NULL);
+    }
+}
+const char *gmpc_plugin_get_name(gmpcPlugin *plug)
+{
+    g_assert(plug->name != NULL);
+    return plug->name;
+}
+void gmpc_plugin_save_yourself(gmpcPlugin *plug)
+{
+    if(plug->save_yourself)
+    {
+        plug->save_yourself();
+    }
+}
+gboolean gmpc_plugin_get_enabled(gmpcPlugin *plug)
+{
+    if(plug->get_enabled)
+    {
+        return plug->get_enabled();
+    }
+    return TRUE;
+}
+
+gchar *gmpc_plugin_get_data_path(gmpcPlugin *plug)
+{
+#ifdef WIN32
+    debug_printf(DEBUG_INFO, "path: %s\n", plug->path);
+    return g_strdup(plug->path);
+#else
+    gchar *url = NULL;
+    gchar **items = g_strsplit(plug->path, G_DIR_SEPARATOR_S, -1);
+    int i = 0;
+    int has_lib = 0;
+    /* count number of items, and check for the lib entry */
+    for(i = 0; items && items[i];i++) {
+        if(strcmp(items[i], "lib")==0) {
+            g_free(items[i]);
+            items[i] = g_strdup("share");
+            has_lib = 1;
+        }
+    }
+    if(has_lib == 1) {
+        gchar *temp = g_build_pathv(G_DIR_SEPARATOR_S, items); 
+        url = g_strdup_printf("%c%s%c", G_DIR_SEPARATOR, temp, G_DIR_SEPARATOR);
+        g_free(temp);
+    }else {
+        /* Ok it is a homedir */
+        url = g_strdup(plug->path);
+    }
+    g_strfreev(items);
+    debug_printf(DEBUG_INFO, "path: %s\n",url);
+    return url;
+#endif
 }
