@@ -1,20 +1,32 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <libmpd/libmpd.h>
+#include <string.h>
+#include <config.h>
 #include "main.h"
 #include "plugin.h"
 #include "gmpc-clicklabel.h"
+#include "vala/gmpc-progress.h"
 #include "misc.h"
+
+
 /* name of config field */
 #define TRAY_ICON2_ID "tray-icon2"
 
-GtkStatusIcon *tray_icon2_gsi = NULL;
+#ifdef EGGTRAYICON
+#include "egg/eggtrayicon.h"
+GtkWidget *tray_icon2_gsi = NULL;
+#else
+GtkStatusIcon 	*tray_icon2_gsi = NULL;
+#endif
+
+static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata);
 /**
  * Tooltip 
  */
-GtkWidget	*tray_icon2_tooltip = NULL;
-GtkWidget 	*tray_icon2_tooltip_pb = NULL;
-guint		tray_icon2_tooltip_timeout = 0;
+GtkWidget			*tray_icon2_tooltip = NULL;
+GtkWidget 			*tray_icon2_tooltip_pb = NULL;
+guint				tray_icon2_tooltip_timeout = 0;
 enum{
 	TI2_AT_TOOLTIP,
 	TI2_AT_UPPER_LEFT,
@@ -31,7 +43,7 @@ void popup_timeout_changed(void);
 void popup_position_changed(GtkComboBox *om);
 void popup_enable_toggled(GtkToggleButton *but);
 void tray_enable_toggled(GtkToggleButton *but);
-
+void tray_icon2_preferences_pm_combo_changed(GtkComboBox *cm, gpointer data);
 /**
  * Tray icon
  */
@@ -39,10 +51,14 @@ void tray_enable_toggled(GtkToggleButton *but);
 gboolean tray_icon2_get_available(void)
 {
 	if(tray_icon2_gsi) {
+#ifdef EGGTRAYICON
+        return GTK_WIDGET_REALIZED(tray_icon2_gsi);
+#else
 		if(gtk_status_icon_is_embedded(tray_icon2_gsi) &&
 				gtk_status_icon_get_visible(tray_icon2_gsi)) {
 			return TRUE;
 		}
+#endif        
 	}
 	return FALSE;
 }
@@ -115,95 +131,7 @@ static void tray_icon2_populate_menu(GtkStatusIcon *gsi,guint button, guint acti
 	gtk_widget_show_all(menu);
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
 }
-
-static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata)
-{
-#if GTK_CHECK_VERSION(2,12,0)
-    char buffer[256];
-    mpd_Song *song = mpd_playlist_get_current_song(connection);
-#endif
-	if(what&(MPD_CST_SONGID))
-	{
-		/** 
-		 * If enabled by user, show the tooltip.
-		 * But only if playing or paused.
-		 *
-		 */
-		if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "show-tooltip", TRUE))
-		{
-			int state = mpd_player_get_state(connection);
-			if(state == MPD_PLAYER_PLAY || state == MPD_PLAYER_PAUSE)
-			{
-				tray_icon2_create_tooltip();
-			}
-		}
-#if GTK_CHECK_VERSION(2,12,0)
-        if(tray_icon2_gsi)
-        {
-            mpd_song_markup(buffer, 256,"[%name%: ][%title%|%shortfile%][ - %artist%]",song);
-            gtk_status_icon_set_tooltip(tray_icon2_gsi,buffer);
-        }
-#endif
-	}
-
-	/* update the progress bar if available */
-	if(what&MPD_CST_ELAPSED_TIME)
-	{
-		if(tray_icon2_tooltip && tray_icon2_tooltip_pb)
-		{
-			int totalTime = mpd_status_get_total_song_time(connection);                                 		
-			int elapsedTime = mpd_status_get_elapsed_song_time(connection);	
-			gdouble progress = elapsedTime/(gdouble)MAX(totalTime,1);
-			gchar*label = g_strdup_printf("%02i:%02i/%02i:%02i", elapsedTime/60, elapsedTime%60,
-					totalTime/60,totalTime%60);
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(tray_icon2_tooltip_pb), RANGE(0,1,progress));
-			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(tray_icon2_tooltip_pb), label);
-			q_free(label);
-		}
-	}
-	if(tray_icon2_gsi == NULL)
-		return;
-
-
-	if(what&MPD_CST_STATE)
-	{
-		int state = mpd_player_get_state(connection);
-		if(state == MPD_PLAYER_PLAY){
-			gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-play");
-#if GTK_CHECK_VERSION(2,12,0)
-            mpd_song_markup(buffer, 256,"[%name%: ][%title%|%shortfile%][ - %artist%]",song);
-            gtk_status_icon_set_tooltip(tray_icon2_gsi,buffer);
-#endif
-		} else if(state == MPD_PLAYER_PAUSE){
-			gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-pause");
-#if GTK_CHECK_VERSION(2,12,0)
-            gtk_status_icon_set_tooltip(tray_icon2_gsi,_("Gnome Music Player Client"));
-#endif
-		} else {
-			gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray");
-#if GTK_CHECK_VERSION(2,12,0)
-            gtk_status_icon_set_tooltip(tray_icon2_gsi,_("Gnome Music Player Client"));
-#endif
-		}
-	}
-}
-/**
- * Show right icon when (dis)connected
- */
-static void tray_icon2_connection_changed(MpdObj *mi, int connect,void *user_data)
-{
-	if(tray_icon2_gsi == NULL)
-		return;
-
-
-
-    if(connect)	{
-		tray_icon2_status_changed(mi, MPD_CST_STATE,NULL);
-	} else {
-		gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-disconnected");
-	}
-}
-
+#ifndef EGGTRAYICON
 static void tray_icon2_embedded_changed(GtkStatusIcon *icon,GParamSpec *arg1, gpointer data)
 {
     if(gtk_status_icon_is_embedded(icon))
@@ -216,18 +144,71 @@ static void tray_icon2_embedded_changed(GtkStatusIcon *icon,GParamSpec *arg1, gp
         create_playlist3();
     }
 }
+#endif
 /**
  * Initialize the tray icon 
  */
+#ifdef EGGTRAYICON
+static int tray_icon2_button_press_event(GtkWidget *tray, GdkEventButton *event, gpointer data)
+{
+    if(event->button == 1)
+    {
+        tray_icon2_activate(NULL, NULL);
+    }
+    else if(event->button == 3)
+    {
+        tray_icon2_populate_menu(NULL, event->button, event->time, NULL);
+    }
+    else if (event->button == 2)
+    {
+       play_song(); 
+    }
+    return TRUE;
+}
+static int tray_icon2_button_scroll_event(GtkWidget *tray, GdkEventScroll *event, gpointer data)
+{
+    if(event->direction == GDK_SCROLL_UP)
+    {
+       volume_up(); 
+    }else if (event->direction == GDK_SCROLL_DOWN) {
+        volume_down();
+    }else if (event->direction == GDK_SCROLL_LEFT) {
+        prev_song();
+    }else if (event->direction == GDK_SCROLL_RIGHT) {
+        next_song();
+    }
+    return TRUE;
+}
+
+static int tray_icon2_button_enter_notify_event(GtkWidget *tray, GdkEventCrossing *event, gpointer data)
+{
+
+    tray_icon2_create_tooltip();
+    return FALSE;
+}
+#endif
 static void tray_icon2_init(void)
 {
+
 	if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "enable", TRUE))
 	{
+#ifdef EGGTRAYICON        
+        tray_icon2_gsi = (GtkWidget *)egg_tray_icon_new("gmpc");
+        gtk_container_add(GTK_CONTAINER(tray_icon2_gsi), gtk_image_new_from_icon_name("gmpc-tray", GTK_ICON_SIZE_MENU));
+        gtk_widget_show_all(tray_icon2_gsi);
+        gtk_widget_add_events(tray_icon2_gsi, GDK_SCROLL_MASK);
+        g_signal_connect(G_OBJECT(tray_icon2_gsi), "button-press-event", G_CALLBACK(tray_icon2_button_press_event), NULL);
+        g_signal_connect(G_OBJECT(tray_icon2_gsi), "scroll-event", G_CALLBACK(tray_icon2_button_scroll_event), NULL);
+
+        g_signal_connect(G_OBJECT(tray_icon2_gsi), "enter-notify-event", G_CALLBACK(tray_icon2_button_enter_notify_event), NULL);
+#else
 		tray_icon2_gsi = gtk_status_icon_new_from_icon_name ("gmpc-tray-disconnected");
+
 		/* connect the (sparse) signals */
 		g_signal_connect(G_OBJECT(tray_icon2_gsi), "popup-menu", G_CALLBACK(tray_icon2_populate_menu), NULL);
 		g_signal_connect(G_OBJECT(tray_icon2_gsi), "activate", G_CALLBACK(tray_icon2_activate), NULL);
         g_signal_connect(G_OBJECT(tray_icon2_gsi), "notify::embedded", G_CALLBACK(tray_icon2_embedded_changed), NULL);
+#endif
 	}
 }
 /**
@@ -237,7 +218,12 @@ static void tray_icon2_destroy(void)
 {
 	if(tray_icon2_gsi)
 	{
+#ifdef EGGTRAYICON
+        gtk_widget_destroy(tray_icon2_gsi);
+#else
 		g_object_unref(tray_icon2_gsi);
+#endif        
+        tray_icon2_gsi = NULL;
 	}
 }
 /**
@@ -256,46 +242,28 @@ static void tray_icon2_set_enabled(int enabled)
 	if(enabled)	{
 		if(!tray_icon2_gsi) {
 			tray_icon2_init();
+            tray_icon2_status_changed(connection, MPD_CST_SONGID, NULL);
 		} else {
+#ifndef EGGTRAYICON
 			gtk_status_icon_set_visible(GTK_STATUS_ICON(tray_icon2_gsi), TRUE);
+#endif            
 		}
 	} else {
 		if(tray_icon2_gsi) {
+#ifdef EGGTRAYICON
+            gtk_widget_destroy(tray_icon2_gsi);
+            tray_icon2_gsi = NULL;
+#else
 			gtk_status_icon_set_visible(GTK_STATUS_ICON(tray_icon2_gsi), FALSE);
+#endif            
 		}		
 	}
 }
 /**
  * TOOLTIP 
  */
-static void tray_icon2_tooltip_song(void)
-{
-	mpd_Song *song = mpd_playlist_get_current_song(connection);
-	if(song)
-	{
-		info2_activate();
-		info2_fill_song_view(song);	
-	}
-}
-static void tray_icon2_tooltip_artist(void)
-{
-	mpd_Song *song = mpd_playlist_get_current_song(connection);
-	if(song && song->artist)
-	{
-		info2_activate();
-		info2_fill_artist_view(song->artist);
-	}
-}
-static void tray_icon2_tooltip_album(void)
-{
-	mpd_Song *song = mpd_playlist_get_current_song(connection);
-	if(song && song->artist && song->album)
-	{
-		info2_activate();
-		info2_fill_album_view(song->artist,song->album);
-	}
-}
-
+static gboolean has_buttons = FALSE;
+static GtkWidget *play_button = NULL;
 
 static gboolean tray_icon2_tooltip_destroy(void)
 {
@@ -308,9 +276,70 @@ static gboolean tray_icon2_tooltip_destroy(void)
 		g_source_remove(tray_icon2_tooltip_timeout);
 	}
 	tray_icon2_tooltip_timeout = 0;
+    has_buttons = FALSE;
 	/* remove the timeout */
 	return FALSE;	
 }
+
+static gboolean tray_icon2_tooltip_button_press_event(GtkWidget *box, GdkEventButton *event, GtkWidget *vbox)
+{
+    if(event->button == 3 && !has_buttons)
+    {
+        GtkWidget *hbox,*button;
+        int	state = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5);
+        if(tray_icon2_tooltip_timeout)
+        {
+            g_source_remove(tray_icon2_tooltip_timeout);
+        }
+        tray_icon2_tooltip_timeout = g_timeout_add_seconds(state*2, (GSourceFunc)tray_icon2_tooltip_destroy, NULL);
+
+        has_buttons = TRUE;
+
+        hbox = gtk_hbox_new(TRUE, 6);
+        /* prev */
+        button = gtk_button_new();
+        gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS,GTK_ICON_SIZE_BUTTON));
+        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+        g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(prev_song), NULL);
+        /* stop */
+        button = gtk_button_new();
+        gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_STOP,GTK_ICON_SIZE_BUTTON));
+        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+        g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(stop_song), NULL);
+        /* pause/play */
+		state = mpd_player_get_state(connection);
+		if(state != MPD_PLAYER_PLAY)
+        {
+            play_button = button = gtk_button_new();
+            gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,GTK_ICON_SIZE_BUTTON));
+            gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+            g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(play_song), NULL);
+        }
+        else
+        {
+            play_button = button = gtk_button_new();
+            gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE,GTK_ICON_SIZE_BUTTON));
+
+            gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+            g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(play_song), NULL);
+        }
+        /* next */
+        button = gtk_button_new();
+        gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_NEXT,GTK_ICON_SIZE_BUTTON));
+
+        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+        g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(next_song), NULL);
+
+        gtk_widget_show_all(hbox);
+        gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE,0);
+
+        return TRUE;
+    }
+    tray_icon2_tooltip_destroy();
+    return TRUE;
+}
+
+
 void tray_icon2_create_tooltip(void)
 {
 	int x=0,y=0,monitor;
@@ -322,40 +351,49 @@ void tray_icon2_create_tooltip(void)
 	GtkWidget *event = NULL;
 	GtkWidget *coverimg = NULL;
 	mpd_Song *song = NULL;
-	int state;
- 
-#if 0 //GTK_CHECK_VERSION(2,12,0)
-	GdkColormap *colormap;
-#endif
+    int tooltip_timeout = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5);
+	int state = 0; 
+	int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
+	int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
+
+
+	song = mpd_playlist_get_current_song(connection);
 	/**
 	 * if the tooltip still exists destroy it... 
 	 */
 	if(tray_icon2_tooltip)
 	{
+        /* Do a check to avoid ugly redraws */
+        mpd_Song *song2 = g_object_get_data(G_OBJECT(tray_icon2_tooltip), "song");
+        if(song2 && song ){
+            if(song->file && song2->file && strcmp(song2->file, song->file) == 0)
+            {
+                if(tray_icon2_tooltip_timeout)
+                {
+                    g_source_remove(tray_icon2_tooltip_timeout);
+                }
+                tray_icon2_tooltip_timeout = g_timeout_add_seconds(tooltip_timeout, (GSourceFunc)tray_icon2_tooltip_destroy, NULL);
+                return;
+            }
+        }
 		tray_icon2_tooltip_destroy();
 	}
 	if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "show-tooltip", 1) == 0)
 		return;
+	/* If gmpc is fullscreen, don't show the tooltip */
+	if(pl3_window_is_fullscreen())
+		return;
 	/*
 	 * 	Creat the tootlip window 
 	 */
-#if 0//GTK_CHECK_VERSION(2,12,0)
-	tray_icon2_tooltip = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_decorated(GTK_WINDOW (tray_icon2_tooltip),FALSE);      
-    gtk_window_set_type_hint (GTK_WINDOW (tray_icon2_tooltip),GDK_WINDOW_TYPE_HINT_NOTIFICATION);
-    gtk_window_stick (GTK_WINDOW (tray_icon2_tooltip));
-
-    gtk_window_set_skip_taskbar_hint(GTK_WINDOW (tray_icon2_tooltip),TRUE);
-    gtk_window_set_keep_above (GTK_WINDOW (tray_icon2_tooltip),TRUE);
-    gtk_window_set_accept_focus(GTK_WINDOW (tray_icon2_tooltip),FALSE);
-	gtk_window_set_focus_on_map(GTK_WINDOW (tray_icon2_tooltip), FALSE);
-#else
 	tray_icon2_tooltip = gtk_window_new(GTK_WINDOW_POPUP);
-#endif
+
+	/* causes the border */
 	gtk_widget_modify_bg(GTK_WIDGET(tray_icon2_tooltip), GTK_STATE_NORMAL, &(pl3_win->style->black));
 	gtk_container_set_border_width(GTK_CONTAINER(tray_icon2_tooltip),1);
 	gtk_window_set_default_size(GTK_WINDOW(tray_icon2_tooltip), 300,-1);
-    gtk_window_set_transient_for(GTK_WINDOW(tray_icon2_tooltip), GTK_WINDOW(pl3_win));
+	gtk_window_set_transient_for(GTK_WINDOW(tray_icon2_tooltip), GTK_WINDOW(pl3_win));
+
 	/*
 	 * Tooltip exists from 2 parts..
 	 * ------------------
@@ -365,7 +403,6 @@ void tray_icon2_create_tooltip(void)
 	 */
 	hbox = gtk_hbox_new(FALSE,0);
 	gtk_container_add(GTK_CONTAINER(tray_icon2_tooltip), hbox);	
-
 	/**
 	 *  1
 	 *
@@ -391,7 +428,6 @@ void tray_icon2_create_tooltip(void)
 	 * Pack the widget in a eventbox so we can set background color 
 	 */
 	event = gtk_event_box_new();
-	g_signal_connect(G_OBJECT(event), "button-press-event", G_CALLBACK(tray_icon2_tooltip_destroy), NULL);
 	gtk_widget_set_size_request(event, 86,86);
 	gtk_widget_modify_bg(GTK_WIDGET(event), GTK_STATE_NORMAL, &(pl3_win->style->bg[GTK_STATE_SELECTED]));
 	gtk_container_add(GTK_CONTAINER(event), coverimg);
@@ -406,81 +442,103 @@ void tray_icon2_create_tooltip(void)
 	 */
 	event = gtk_event_box_new();
 	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_widget_modify_bg(GTK_WIDGET(event), GTK_STATE_NORMAL, &(pl3_win->style->white));
+	g_signal_connect(G_OBJECT(hbox), "button-press-event", G_CALLBACK(tray_icon2_tooltip_button_press_event), vbox);
+
+	gtk_widget_modify_bg(GTK_WIDGET(event), GTK_STATE_NORMAL, &(pl3_win->style->light[GTK_STATE_NORMAL]));
 	gtk_container_set_border_width(GTK_CONTAINER(vbox),3);
 	gtk_container_add(GTK_CONTAINER(event), vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), event, TRUE,TRUE,0);
 	/**
 	 * If there is a song, show show song info
 	 */
-	song = mpd_playlist_get_current_song(connection);
 	if(song)
 	{
-		int size = cfg_get_single_value_as_int_with_default(config, "tray-icon2", "size-offset", 0);
+		int size_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "size-offset", 1024);
+		size_offset = (size_offset < 100)?1024:size_offset;
+
 		/** Artist label */
 		if(song->title || song->file || song->name)
 		{
 			char buffer[256];
-			mpd_song_markup(buffer, 256,"[%title%|%shortfile%][ (%name%)]",song);
-			label = gmpc_clicklabel_new(buffer);
-			g_signal_connect(G_OBJECT(label), "button-press-event", G_CALLBACK(tray_icon2_tooltip_song), NULL);
-			gmpc_clicklabel_set_do_bold(GMPC_CLICKLABEL(label),FALSE);
-			gmpc_clicklabel_font_size(GMPC_CLICKLABEL(label),size+3);
+			gchar *test = g_strdup_printf("<span size='%i' weight='bold'>[%%title%%|%%shortfile%%][ (%%name%%)]</span>", 14*size_offset);
+			mpd_song_markup_escaped(buffer, 256,test,song);
+			q_free(test);
+			label = gtk_label_new("");
+			gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
+			gtk_misc_set_alignment(GTK_MISC(label), 0,0.5);
+			gtk_label_set_markup(GTK_LABEL(label), buffer);
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE,FALSE,0);
-            gtk_widget_modify_fg(GMPC_CLICKLABEL(label)->label, GTK_STATE_NORMAL, &(pl3_win->style->black));
 		}
 		if(song->artist)
 		{
-			label = gmpc_clicklabel_new(song->artist);
-            gtk_widget_modify_fg(GMPC_CLICKLABEL(label)->label, GTK_STATE_NORMAL, &(pl3_win->style->black));
-            g_signal_connect(G_OBJECT(label), "button-press-event", G_CALLBACK(tray_icon2_tooltip_artist), NULL);
-			gmpc_clicklabel_set_do_bold(GMPC_CLICKLABEL(label),FALSE);
-			gmpc_clicklabel_font_size(GMPC_CLICKLABEL(label),size);
+			char buffer[256];
+			gchar *test = g_strdup_printf("<span size='%i'>%%artist%%</span>", 10*size_offset);
+			label = gtk_label_new("");
+			gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
+			mpd_song_markup_escaped(buffer, 256,test,song);
+			q_free(test);
+			gtk_label_set_markup(GTK_LABEL(label), buffer);
+			gtk_misc_set_alignment(GTK_MISC(label), 0,0.5);
+
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE,FALSE,0);
 		}
 		if(song->album)
 		{
-			label = gmpc_clicklabel_new(song->album);
-            gtk_widget_modify_fg(GMPC_CLICKLABEL(label)->label, GTK_STATE_NORMAL, &(pl3_win->style->black));
-			g_signal_connect(G_OBJECT(label), "button-press-event", G_CALLBACK(tray_icon2_tooltip_album), NULL);
-			gmpc_clicklabel_set_do_bold(GMPC_CLICKLABEL(label),FALSE);
-			gmpc_clicklabel_font_size(GMPC_CLICKLABEL(label),size-3);
+			char buffer[256];
+			gchar *test = g_strdup_printf("<span size='%i'>%%album%%[ (%%year%%)]</span>", 8*size_offset);
+			label = gtk_label_new("");
+			gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);           
+			mpd_song_markup_escaped(buffer, 256,test,song);
+			q_free(test);
+			gtk_misc_set_alignment(GTK_MISC(label), 0,0.5);
+			gtk_label_set_markup(GTK_LABEL(label), buffer);
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE,FALSE,0);
 		}
-		tray_icon2_tooltip_pb = gtk_progress_bar_new();
+		tray_icon2_tooltip_pb = (GtkWidget *) gmpc_progress_new();
 		/* Update the progressbar */
-		{
-			int totalTime = mpd_status_get_total_song_time(connection);                                 		
-			int elapsedTime = mpd_status_get_elapsed_song_time(connection);	
-			gdouble progress = elapsedTime/(gdouble)MAX(totalTime,1);
-			gchar*label = g_strdup_printf("%02i:%02i/%02i:%02i", elapsedTime/60, elapsedTime%60,
-					totalTime/60,totalTime%60);
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(tray_icon2_tooltip_pb), RANGE(0,1,progress));
-			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(tray_icon2_tooltip_pb), label);
-			q_free(label);
-		}
-
+        gmpc_progress_set_time(GMPC_PROGRESS(tray_icon2_tooltip_pb), 
+                mpd_status_get_total_song_time(connection),
+                mpd_status_get_elapsed_song_time(connection));
 
+        gtk_widget_modify_bg(GTK_WIDGET(tray_icon2_tooltip_pb), GTK_STATE_NORMAL, &(pl3_win->style->light[GTK_STATE_NORMAL]));
+        g_object_set_data_full(G_OBJECT(tray_icon2_tooltip), "song", mpd_songDup(song),(GDestroyNotify)mpd_freeSong); 
 		gtk_box_pack_start(GTK_BOX(vbox), tray_icon2_tooltip_pb, TRUE,FALSE,0);
 	} else {
+		gchar *value = g_markup_printf_escaped("<span size='large'>%s</span>", _("Gnome Music Player Client"));
 		label = gtk_label_new("");
-		gtk_label_set_markup(GTK_LABEL(label), _("<span size='large'>Gnome Music Player Client</span>"));
+		gtk_label_set_markup(GTK_LABEL(label),value);
 		gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
 		gtk_box_pack_start(GTK_BOX(vbox), label, TRUE,TRUE,0);
-
+		g_free(value);
 	}
 	/**
 	 * 	Position the popup
 	 */
 	state = cfg_get_single_value_as_int_with_default(config,TRAY_ICON2_ID, "tooltip-position", TI2_AT_TOOLTIP);
-	if(state == TI2_AT_TOOLTIP && tray_icon2_get_available()) {
+	if(state == TI2_AT_TOOLTIP && tray_icon2_get_available()) 
+	{
 
 		GdkRectangle rect, rect2;
 		GtkOrientation orientation;
-        int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
-        int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
+#ifdef EGGTRAYICON
+        GdkWindow *w = NULL;
+        gtk_widget_realize(tray_icon2_gsi);
+#if GTK_CHECK_VERSION(2,14,0)        
+        w = gtk_widget_get_window(tray_icon2_gsi);
+#else
+        w = GTK_WIDGET(tray_icon2_gsi)->window;
+#endif
+        screen = gtk_widget_get_screen(tray_icon2_gsi);
+        orientation = egg_tray_icon_get_orientation((EggTrayIcon *)tray_icon2_gsi);
+        rect.width = tray_icon2_gsi->allocation.width; 
+        rect.height = tray_icon2_gsi->allocation.height; 
+        if(w)
+        {
+            gdk_window_get_origin(w, &(rect.x), &(rect.y));
+#else
 		if(gtk_status_icon_get_geometry(tray_icon2_gsi, &screen, &rect, &orientation))
-		{
+        {
+#endif        
 			monitor  = gdk_screen_get_monitor_at_point(screen, rect.x, rect.y);
 			gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
 			/* Get Y */
@@ -509,73 +567,167 @@ void tray_icon2_create_tooltip(void)
 			}
 		}
 		gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), rect2.x+x,rect2.y+y);
-	} else if (state == TI2_AT_UPPER_LEFT) {
-       int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
-        int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
-		screen =gtk_widget_get_screen(pl3_win);
-		GdkRectangle rect2;
-		monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
-		gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
-		gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), rect2.x+5+x_offset,rect2.y+5+y_offset);
-	} else if (state == TI2_AT_UPPER_RIGHT) {
-        int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
-        int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
-		screen =gtk_widget_get_screen(pl3_win);
-		GdkRectangle rect2;
-		monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
-		gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
-		/** Set Y = 0; */
-		y = rect2.y+5;
-		/** X is upper right - width */
-		x = rect2.x+rect2.width-5-300;
-		gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
-	} else if (state == TI2_AT_LOWER_LEFT) {
-        int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
-        int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
-		screen =gtk_widget_get_screen(pl3_win);
-		GdkRectangle rect2;
-		monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
-		gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
-		/** Set Y = window height - size; */
-		y = rect2.y+rect2.height-5-95;
-		/** X =5 */ 
-		x = rect2.x+ 5; 
-		gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
-	} else {
-        int x_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "x-offset", 0);
-        int y_offset = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "y-offset", 0);
-		screen =gtk_widget_get_screen(pl3_win);
-		GdkRectangle rect2;
-		monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
-		gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
-		/** Set Y = window height - size; */
-		y = rect2.y+rect2.height-5-95;
-		/** X =window width - width */ 
-		x = rect2.x+rect2.width-5-300; 
-		gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
-	}
-	/**
-	 * Show the tooltip
-	 */
-/* do some stuff to allow transparency */
-#if 0 //GTK_CHECK_VERSION(2,12,0)
-    screen = gtk_window_get_screen(GTK_WINDOW(tray_icon2_tooltip));
-    colormap = gdk_screen_get_rgba_colormap(screen);
-	if (colormap != NULL && gdk_screen_is_composited(screen))
+	} else 
+		if (state == TI2_AT_UPPER_LEFT) 
+		{
+			GdkRectangle rect2;
+			screen =gtk_widget_get_screen(pl3_win);
+			
+			monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
+			gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
+			gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), rect2.x+5+x_offset,rect2.y+5+y_offset);
+		} else if (state == TI2_AT_UPPER_RIGHT)  {
+			GdkRectangle rect2;
+			screen =gtk_widget_get_screen(pl3_win);
+			
+			monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
+			gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
+			/** Set Y = 0; */
+			y = rect2.y+5;
+			/** X is upper right - width */
+			x = rect2.x+rect2.width-5-300;
+			gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
+		} else if (state == TI2_AT_LOWER_LEFT) 
+		{
+			GdkRectangle rect2;
+			screen =gtk_widget_get_screen(pl3_win);
+
+			monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
+			gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
+			/** Set Y = window height - size; */
+			y = rect2.y+rect2.height-5-95;
+			/** X =5 */ 
+			x = rect2.x+ 5; 
+			gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
+		} else 
+		{
+			GdkRectangle rect2;
+			screen =gtk_widget_get_screen(pl3_win);
+			
+			monitor  = gdk_screen_get_monitor_at_window(screen, pl3_win->window);
+			gdk_screen_get_monitor_geometry(screen, monitor, &rect2);
+			/** Set Y = window height - size; */
+			y = rect2.y+rect2.height-5-95;
+			/** X =window width - width */ 
+			x = rect2.x+rect2.width-5-300; 
+			gtk_window_move(GTK_WINDOW(tray_icon2_tooltip), x+x_offset,y+y_offset);
+		}
+
+		gtk_widget_show_all(tray_icon2_tooltip);
+
+
+		/**
+		 * Destroy it after 5 seconds
+		 */
+		tray_icon2_tooltip_timeout = g_timeout_add_seconds(tooltip_timeout, (GSourceFunc)tray_icon2_tooltip_destroy, NULL);
+}
+
+static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata)
+{
+	char buffer[256];
+	mpd_Song *song = mpd_playlist_get_current_song(connection);
+	if(what&(MPD_CST_SONGID))
 	{
-		gtk_widget_set_colormap(tray_icon2_tooltip, colormap);
-        gtk_window_set_opacity(GTK_WINDOW(tray_icon2_tooltip), 0.8);
-	}	    
+		/** 
+		 * If enabled by user, show the tooltip.
+		 * But only if playing or paused.
+		 *
+		 */
+		if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "show-tooltip", TRUE))
+		{
+			int state = mpd_player_get_state(connection);
+			if(state == MPD_PLAYER_PLAY || state == MPD_PLAYER_PAUSE)
+			{
+				tray_icon2_create_tooltip();
+			}
+		}
+		if(tray_icon2_gsi)
+		{
+			mpd_song_markup(buffer, 256,"[%name%: ][%title%|%shortfile%][ - %artist%]",song);
+#ifdef EGGTRAYICON
+            gtk_widget_set_tooltip_text(tray_icon2_gsi, buffer);
+#else
+			gtk_status_icon_set_tooltip(tray_icon2_gsi,buffer);
+#endif            
+		}
+	}
 
-#endif
-	gtk_widget_show_all(tray_icon2_tooltip);
+	/* update the progress bar if available */
+	if(what&MPD_CST_ELAPSED_TIME)
+	{
+		if(tray_icon2_tooltip && tray_icon2_tooltip_pb)
+		{
+            gmpc_progress_set_time(GMPC_PROGRESS(tray_icon2_tooltip_pb),
+                    mpd_status_get_total_song_time(connection),                                 		
+                    mpd_status_get_elapsed_song_time(connection));	
+        }
+	}
+	if(tray_icon2_gsi == NULL)
+		return;
 
 
-	/**
-	 * Destroy it after 5 seconds
-	 */
-	state = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5);
-	tray_icon2_tooltip_timeout = g_timeout_add(state*1000, (GSourceFunc)tray_icon2_tooltip_destroy, NULL);
+	if(what&MPD_CST_STATE)
+	{
+		int state = mpd_player_get_state(connection);
+		if(state == MPD_PLAYER_PLAY){
+			mpd_song_markup(buffer, 256,"[%name%: ][%title%|%shortfile%][ - %artist%]",song);
+#ifdef EGGTRAYICON
+            gtk_image_set_from_icon_name(GTK_IMAGE(gtk_bin_get_child(GTK_BIN(tray_icon2_gsi))), "gmpc-tray-play", GTK_ICON_SIZE_MENU);
+            gtk_widget_set_tooltip_text(tray_icon2_gsi, buffer);
+#else
+            gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-play");
+            gtk_status_icon_set_tooltip(tray_icon2_gsi,buffer);
+#endif            
+			if(has_buttons)
+			{
+				gtk_button_set_image(GTK_BUTTON(play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_BUTTON));
+			}
+		} else if(state == MPD_PLAYER_PAUSE){
+			
+#ifdef EGGTRAYICON
+            gtk_image_set_from_icon_name(GTK_IMAGE(gtk_bin_get_child(GTK_BIN(tray_icon2_gsi))), "gmpc-tray-pause", GTK_ICON_SIZE_MENU);
+            gtk_widget_set_tooltip_text(tray_icon2_gsi, _("Gnome Music Player Client"));
+#else
+            gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-pause");
+            gtk_status_icon_set_tooltip(tray_icon2_gsi,_("Gnome Music Player Client"));
+#endif  
+			if(has_buttons)
+			{
+				gtk_button_set_image(GTK_BUTTON(play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
+			}
+		} else {
+			
+#ifdef EGGTRAYICON
+            gtk_image_set_from_icon_name(GTK_IMAGE(gtk_bin_get_child(GTK_BIN(tray_icon2_gsi))), "gmpc-tray", GTK_ICON_SIZE_MENU);
+            gtk_widget_set_tooltip_text(tray_icon2_gsi, _("Gnome Music Player Client"));
+#else
+            gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray");
+            gtk_status_icon_set_tooltip(tray_icon2_gsi,_("Gnome Music Player Client"));
+#endif  
+			if(has_buttons)
+			{
+				gtk_button_set_image(GTK_BUTTON(play_button), gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_BUTTON));
+			}
+		}
+	}
+}
+/**
+ * Show right icon when (dis)connected
+ */
+static void tray_icon2_connection_changed(MpdObj *mi, int connect,void *user_data)
+{
+	if(tray_icon2_gsi == NULL)
+		return;
+
+    if(connect)	{
+		tray_icon2_status_changed(mi, MPD_CST_STATE,NULL);
+	} else {
+#ifdef EGGTRAYICON
+        gtk_image_set_from_icon_name(GTK_IMAGE(gtk_bin_get_child(GTK_BIN(tray_icon2_gsi))), "gmpc-tray-disconnected", GTK_ICON_SIZE_MENU);
+#else
+		gtk_status_icon_set_from_icon_name(tray_icon2_gsi, "gmpc-tray-disconnected");
+#endif        
+	}
 }
 
 /**
@@ -595,7 +747,7 @@ void tray_enable_toggled(GtkToggleButton *but)
 }
 
 /* this sets all the settings in the notification area preferences correct */
-static void tray_update_settings()
+static void tray_update_settings(void)
 {
 	gtk_toggle_button_set_active((GtkToggleButton *)
 			glade_xml_get_widget(tray_icon2_preferences_xml, "ck_tray_enable"), 
@@ -619,7 +771,7 @@ void popup_timeout_changed(void)
 			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget(tray_icon2_preferences_xml, "popup_timeout"))));
 }
 
-static void update_popup_settings()
+static void update_popup_settings(void)
 {
 	gtk_toggle_button_set_active((GtkToggleButton *)
 			glade_xml_get_widget(tray_icon2_preferences_xml, "ck_popup_enable"),
